from qds_sdk import util


class Engine:
    '''
    Use this class to set engine config settings of cluster
    qds_sdk.engine.Engine is the class which stores information about engine config settings.
    You can use objects of this class to set engine_config settings while create/update/clone a cluster.
    '''

    def __init__(self, flavour=None):
        self.flavour = flavour
        self.hadoop_settings = {}
        self.hive_settings = {}
        self.presto_settings = {}
        self.spark_settings = {}
        self.airflow_settings = {}
        self.engine_config = {}
        self.mlflow_settings = {}

    def set_engine_config(self,
                          custom_hadoop_config=None,
                          use_qubole_placement_policy=None,
                          fairscheduler_config_xml=None,
                          default_pool=None,
                          presto_version=None,
                          custom_presto_config=None,
                          spark_version=None,
                          custom_spark_config=None,
                          hive_version=None,
                          dbtap_id=None,
                          fernet_key=None,
                          overrides=None,
                          airflow_version=None,
                          airflow_python_version=None,
                          is_ha=None,
                          enable_rubix=None,
                          mlflow_version=None,
                          mlflow_dbtap_id=None):
        '''

        Args:
            custom_hadoop_config: Custom Hadoop configuration overrides.

            use_qubole_placement_policy: Use Qubole Block Placement policy for
                clusters with spot nodes.

            fairscheduler_config_xml: XML string with custom configuration
                parameters for the fair scheduler.

            default_pool: The default pool for the fair scheduler.

            presto_version: Version of presto to be used in cluster

            custom_presto_config: Custom Presto configuration overrides.

            spark_version: Version of spark to be used in cluster

            custom_spark_config: Specify the custom Spark configuration overrides

            hive_version: Version of hive to be used in cluster

            dbtap_id: ID of the data store inside QDS

            fernet_key: Encryption key for sensitive information inside airflow database.
                For example, user passwords and connections. It must be a 32 url-safe base64 encoded bytes.

            overrides: Airflow configuration to override the default settings.Use the following syntax for overrides:
                <section>.<property>=<value>\n<section>.<property>=<value>...

            airflow_version: The airflow version.

            airflow_python_version: The python version for the environment on the cluster.

            is_ha: Enabling HA config for cluster
            is_deeplearning : this is a deeplearning cluster config
            enable_rubix: Enable rubix on the cluster
             mlflow_version : this is the version of the mlflow cluster

        '''

        self.set_hadoop_settings(custom_hadoop_config, use_qubole_placement_policy,
                                 is_ha, fairscheduler_config_xml,
                                 default_pool, enable_rubix)
        self.set_hive_settings(hive_version)
        self.set_presto_settings(presto_version, custom_presto_config)
        self.set_spark_settings(spark_version, custom_spark_config)
        self.set_airflow_settings(dbtap_id, fernet_key, overrides, airflow_version, airflow_python_version)
        self.set_mlflow_settings(mlflow_version, mlflow_dbtap_id)

    def set_fairscheduler_settings(self,
                                   fairscheduler_config_xml=None,
                                   default_pool=None):
        self.hadoop_settings['fairscheduler_settings'] = {}
        self.hadoop_settings['fairscheduler_settings']['fairscheduler_config_xml'] = \
            fairscheduler_config_xml
        self.hadoop_settings['fairscheduler_settings']['default_pool'] = default_pool

    def set_hadoop_settings(self,
                            custom_hadoop_config=None,
                            use_qubole_placement_policy=None,
                            is_ha=None,
                            fairscheduler_config_xml=None,
                            default_pool=None,
                            enable_rubix=None):
        self.hadoop_settings['custom_hadoop_config'] = custom_hadoop_config
        self.hadoop_settings['use_qubole_placement_policy'] = use_qubole_placement_policy
        self.hadoop_settings['is_ha'] = is_ha
        self.set_fairscheduler_settings(fairscheduler_config_xml, default_pool)
        self.hadoop_settings['enable_rubix'] = enable_rubix

    def set_hive_settings(self,
                          hive_version=None):
        self.hive_settings['hive_version'] = hive_version

    def set_presto_settings(self,
                            presto_version=None,
                            custom_presto_config=None):
        self.presto_settings['presto_version'] = presto_version
        self.presto_settings['custom_presto_config'] = custom_presto_config

    def set_spark_settings(self,
                           spark_version=None,
                           custom_spark_config=None):
        self.spark_settings['spark_version'] = spark_version
        self.spark_settings['custom_spark_config'] = custom_spark_config

    def set_airflow_settings(self,
                             dbtap_id=None,
                             fernet_key=None,
                             overrides=None,
                             airflow_version="1.10.0",
                             airflow_python_version="2.7"):
        self.airflow_settings['dbtap_id'] = dbtap_id
        self.airflow_settings['fernet_key'] = fernet_key
        self.airflow_settings['overrides'] = overrides
        self.airflow_settings['version'] = airflow_version
        self.airflow_settings['airflow_python_version'] = airflow_python_version

    def set_mlflow_settings(self,
                            mlflow_version="1.7",
                            mlflow_dbtap_id=None):
        self.mlflow_settings['version'] = mlflow_version
        self.mlflow_settings['dbtap_id'] = mlflow_dbtap_id

    def set_engine_config_settings(self, arguments):
        custom_hadoop_config = util._read_file(arguments.custom_hadoop_config_file)
        fairscheduler_config_xml = util._read_file(arguments.fairscheduler_config_xml_file)
        custom_presto_config = util._read_file(arguments.presto_custom_config_file)
        is_deeplearning = False

        self.set_engine_config(custom_hadoop_config=custom_hadoop_config,
                               use_qubole_placement_policy=arguments.use_qubole_placement_policy,
                               fairscheduler_config_xml=fairscheduler_config_xml,
                               default_pool=arguments.default_pool,
                               presto_version=arguments.presto_version,
                               custom_presto_config=custom_presto_config,
                               spark_version=arguments.spark_version,
                               custom_spark_config=arguments.custom_spark_config,
                               hive_version=arguments.hive_version,
                               dbtap_id=arguments.dbtap_id,
                               fernet_key=arguments.fernet_key,
                               overrides=arguments.overrides,
                               airflow_version=arguments.airflow_version,
                               airflow_python_version=arguments.airflow_python_version,
                               enable_rubix=arguments.enable_rubix,
                               mlflow_version=arguments.mlflow_version,
                               mlflow_dbtap_id=arguments.mlflow_dbtap_id)

    @staticmethod
    def engine_parser(argparser):
        engine_group = argparser.add_argument_group("engine settings")
        engine_group.add_argument("--flavour",
                                  dest="flavour",
                                  choices=["hadoop", "hadoop2", "hs2", "hive", "presto", "spark", "sparkstreaming",
                                           "hbase", "airflow", "deeplearning", "mlflow"],
                                  default=None,
                                  help="Set engine flavour")

        hadoop_settings_group = argparser.add_argument_group("hadoop settings")
        hadoop_settings_group.add_argument("--custom-hadoop-config",
                                           dest="custom_hadoop_config_file",
                                           default=None,
                                           help="location of file containing custom" +
                                                " hadoop configuration overrides")
        qubole_placement_policy_group = hadoop_settings_group.add_mutually_exclusive_group()
        qubole_placement_policy_group.add_argument("--use-qubole-placement-policy",
                                                   dest="use_qubole_placement_policy",
                                                   action="store_true",
                                                   default=None,
                                                   help="Use Qubole Block Placement policy" +
                                                        " for clusters with spot nodes", )
        qubole_placement_policy_group.add_argument("--no-use-qubole-placement-policy",
                                                   dest="use_qubole_placement_policy",
                                                   action="store_false",
                                                   default=None,
                                                   help="Do not use Qubole Block Placement policy" +
                                                        " for clusters with spot nodes", )
        enable_rubix_group = hadoop_settings_group.add_mutually_exclusive_group()
        enable_rubix_group.add_argument("--enable-rubix",
                                        dest="enable_rubix",
                                        action="store_true",
                                        default=None,
                                        help="Enable rubix for cluster", )
        enable_rubix_group.add_argument("--no-enable-rubix",
                                        dest="enable_rubix",
                                        action="store_false",
                                        default=None,
                                        help="Do not enable rubix for cluster", )

        fairscheduler_group = argparser.add_argument_group(
            "fairscheduler configuration options")
        fairscheduler_group.add_argument("--fairscheduler-config-xml",
                                         dest="fairscheduler_config_xml_file",
                                         help="location for file containing" +
                                              " xml with custom configuration" +
                                              " for the fairscheduler", )
        fairscheduler_group.add_argument("--fairscheduler-default-pool",
                                         dest="default_pool",
                                         help="default pool for the" +
                                              " fairscheduler", )

        presto_settings_group = argparser.add_argument_group("presto settings")
        presto_settings_group.add_argument("--presto-version",
                                           dest="presto_version",
                                           default=None,
                                           help="Version of presto for this cluster", )
        presto_settings_group.add_argument("--presto-custom-config",
                                           dest="presto_custom_config_file",
                                           help="location of file containg custom" +
                                                " presto configuration overrides")
        hive_settings_group = argparser.add_argument_group("hive version settings")
        hive_settings_group.add_argument("--hive_version",
                                         dest="hive_version",
                                         default=None,
                                         help="Version of hive for the cluster",)

        spark_settings_group = argparser.add_argument_group("spark settings")
        spark_settings_group.add_argument("--spark-version",
                                          dest="spark_version",
                                          default=None,
                                          help="Version of spark for the cluster", )
        spark_settings_group.add_argument("--custom-spark-config",
                                          dest="custom_spark_config",
                                          default=None,
                                          help="Custom config spark for this cluster", )

        airflow_settings_group = argparser.add_argument_group("airflow settings")
        airflow_settings_group.add_argument("--dbtap-id",
                                            dest="dbtap_id",
                                            default=None,
                                            help="dbtap id for airflow cluster", )
        airflow_settings_group.add_argument("--fernet-key",
                                            dest="fernet_key",
                                            default=None,
                                            help="fernet key for airflow cluster", )
        airflow_settings_group.add_argument("--overrides",
                                            dest="overrides",
                                            default=None,
                                            help="overrides for airflow cluster", )
        airflow_settings_group.add_argument("--airflow-version",
                                            dest="airflow_version",
                                            default=None,
                                            help="airflow version for airflow cluster", )
        airflow_settings_group.add_argument("--airflow-python-version",
                                            dest="airflow_python_version",
                                            default=None,
                                            help="python environment version for airflow cluster", )

        mlflow_settings_group = argparser.add_argument_group("mlflow settings")

        mlflow_settings_group.add_argument("--mlflow-version",
                                           dest="mlflow_version",
                                           default=None,
                                           help="mlflow version for mlflow cluster", )
        mlflow_settings_group.add_argument("--mlflow-dbtap-id",
                                           dest="mlflow_dbtap_id",
                                           default=None,
                                           help="dbtap id for mlflow cluster", )




"""
The sensors module contains the base definition for a generic
sensor call and the implementation of all the specific sensors
"""

from __future__ import print_function
from qds_sdk.qubole import Qubole
from qds_sdk.resource import Resource
from argparse import ArgumentParser

import logging
import json

log = logging.getLogger("qds_sensors")


class SensorCmdLine:

    @staticmethod
    def check(sensor_class, args):
        """
        Method to call Sensor.check after parsing args from cmdline
        :param sensor_class: sensor class
        :param args: inline arguments
        :return: True or False
        """
        parser = SensorCmdLine.parsers(sensor_class)
        parsed = parser.parse_args(args)
        return sensor_class.check(json.loads(parsed.data))

    @staticmethod
    def parsers(sensor_class):
        argparser = ArgumentParser(prog=sensor_class.usage, description=sensor_class.description)
        subparsers = argparser.add_subparsers()

        #Check
        check = subparsers.add_parser("check", help="Check a Sensor")
        check.add_argument("-d", "--data", dest="data", required=True,
                           help="String containing a valid json object")
        check.set_defaults(func=Sensor.check)
        return argparser


class Sensor(Resource):
    """
    qds_sdk.Sensor is the base Qubole sensor class. Different types of Qubole
    sensors can subclass this.
    """

    @classmethod
    def check(cls, data):
        """
        Method to call the sensors api with json payload
        :param data: valid json object
        :return: True or False
        """
        conn = Qubole.agent()
        return conn.post(cls.rest_entity_path, data=data)['status']


class FileSensor(Sensor):
    rest_entity_path = "sensors/file_sensor"

    usage = ("qds.py filesensor check -d 'json string'")
    description = "File Sensor client for Qubole Data Services"


class PartitionSensor(Sensor):
    rest_entity_path = "sensors/partition_sensor"

    usage = ("qds.py partitionsensor check -d 'json string'")
    description = "Hive Partition Sensor client for Qubole Data Services"






from qds_sdk.cloud.cloud import Cloud
class AzureCloud(Cloud):
    '''
    qds_sdk.cloud.AzureCloud is the class which stores information about azure cloud config settings.
    The objects of this class can be use to set azure cloud_config settings while create/update/clone a cluster.
    '''

    def __init__(self):
        self.compute_config = {}
        self.location = {}
        self.network_config = {}
        self.storage_config = {}

    def set_cloud_config(self,
                         compute_client_id=None,
                         compute_client_secret=None,
                         compute_subscription_id=None,
                         compute_tenant_id=None,
                         use_account_compute_creds=None,
                         location=None,
                         storage_access_key=None,
                         storage_account_name=None,
                         disk_storage_account_name=None,
                         disk_storage_account_resource_group_name=None,
                         persistent_security_groups=None,
                         bastion_node_public_dns=None,
                         vnet_name=None,
                         subnet_name=None,
                         vnet_resource_group_name=None,
                         master_elastic_ip=None,
                         master_static_nic_name=None,
                         master_static_public_ip_name=None,
                         resource_group_name=None):
        '''

        Args:
            compute_client_id: Client id for azure cluster

            compute_client_secret: Client secret key for azure cluster

            compute_subscription_id: Subscription id for azure cluster

            compute_tenant_id: Tenant id for azure cluster

            use_account_compute_creds: Set it to true to use the account's compute
                credentials for all clusters of the account.The default value is false

            location: Location for azure cluster

            storage_access_key: Storage access key for azure cluster

            storage_account_name: Storage account name for azure cluster

            disk_storage_account_name: Disk storage account name for azure cluster

            disk_storage_account_resource_group_name: Disk storage account resource group
                namefor azure cluster

            persistent_security_groups: security group to associate with each node of the cluster.
                Typically used to provide access to external hosts

            bastion_node_public_dns: public dns name of the bastion node.
                Required only if cluster is in private subnet of a EC2-VPC

            vnet_name: vnet name for azure

            subnet_name: subnet name for azure

            vnet_resource_group_name: vnet resource group name for azure

            master_elastic_ip: It is the Elastic IP address for attaching to the cluster master

            master_static_nic_name: Name of NIC that has to be attached to cluster's master node

            master_static_public_ip_name: Name of Static Public Ip address that has to be attached
                to cluster's master node

            resource_group_name: Resource group for cluster
        '''

        self.set_compute_config(use_account_compute_creds, compute_tenant_id,
                                compute_subscription_id, compute_client_id,
                                compute_client_secret)
        self.set_location(location)
        self.set_network_config(bastion_node_public_dns, persistent_security_groups,
                                master_elastic_ip, vnet_name, subnet_name,
                                vnet_resource_group_name, master_static_nic_name, master_static_public_ip_name)
        self.set_storage_config(storage_access_key, storage_account_name,
                                disk_storage_account_name,
                                disk_storage_account_resource_group_name)
        self.resource_group_name = resource_group_name

    def set_compute_config(self,
                           use_account_compute_creds=None,
                           compute_tenant_id=None,
                           compute_subscription_id=None,
                           compute_client_id=None,
                           compute_client_secret=None):
        self.compute_config['use_account_compute_creds'] = use_account_compute_creds
        self.compute_config['compute_tenant_id'] = compute_tenant_id
        self.compute_config['compute_subscription_id'] = compute_subscription_id
        self.compute_config['compute_client_id'] = compute_client_id
        self.compute_config['compute_client_secret'] = compute_client_secret

    def set_location(self,
                     location=None):
        self.location['location'] = location

    def set_network_config(self,
                           bastion_node_public_dns=None,
                           persistent_security_groups=None,
                           master_elastic_ip=None,
                           vnet_name=None,
                           subnet_name=None,
                           vnet_resource_group_name=None,
                           master_static_nic_name=None,
                           master_static_public_ip_name=None):
        self.network_config['bastion_node_public_dns'] = bastion_node_public_dns
        self.network_config['persistent_security_groups'] = persistent_security_groups
        self.network_config['master_elastic_ip'] = master_elastic_ip
        self.network_config['vnet_name'] = vnet_name
        self.network_config['subnet_name'] = subnet_name
        self.network_config['vnet_resource_group_name'] = vnet_resource_group_name
        self.network_config['master_static_nic_name'] = master_static_nic_name
        self.network_config['master_static_public_ip_name'] = master_static_public_ip_name

    def set_storage_config(self,
                           storage_access_key=None,
                           storage_account_name=None,
                           disk_storage_account_name=None,
                           disk_storage_account_resource_group_name=None):
        self.storage_config['storage_access_key'] = storage_access_key
        self.storage_config['storage_account_name'] = storage_account_name
        self.storage_config['disk_storage_account_name'] = disk_storage_account_name
        self.storage_config['disk_storage_account_resource_group_name'] \
            = disk_storage_account_resource_group_name

    def set_cloud_config_from_arguments(self, arguments):
        self.set_cloud_config(compute_client_id=arguments.compute_client_id,
                              compute_client_secret=arguments.compute_client_secret,
                              compute_subscription_id=arguments.compute_subscription_id,
                              compute_tenant_id=arguments.compute_tenant_id,
                              use_account_compute_creds=arguments.use_account_compute_creds,
                              location=arguments.location,
                              storage_access_key=arguments.storage_access_key,
                              storage_account_name=arguments.storage_account_name,
                              disk_storage_account_name=arguments.disk_storage_account_name,
                              disk_storage_account_resource_group_name=arguments.disk_storage_account_resource_group_name,
                              vnet_name=arguments.vnet_name,
                              subnet_name=arguments.subnet_name,
                              vnet_resource_group_name=arguments.vnet_resource_group_name,
                              master_static_nic_name=arguments.master_static_nic_name,
                              master_static_public_ip_name=arguments.master_static_public_ip_name,
                              resource_group_name=arguments.resource_group_name)

    def create_parser(self, argparser):
        # compute settings parser
        compute_config = argparser.add_argument_group("compute config settings")
        compute_creds = compute_config.add_mutually_exclusive_group()
        compute_creds.add_argument("--enable-account-compute-creds",
                                   dest="use_account_compute_creds",
                                   action="store_true",
                                   default=None,
                                   help="to use account compute credentials")
        compute_creds.add_argument("--disable-account-compute-creds",
                                   dest="use_account_compute_creds",
                                   action="store_false",
                                   default=None,
                                   help="to disable account compute credentials")
        compute_config.add_argument("--compute-client-id",
                                    dest="compute_client_id",
                                    default=None,
                                    help="client id for azure cluster")
        compute_config.add_argument("--compute-client-secret",
                                    dest="compute_client_secret",
                                    default=None,
                                    help="client secret key for azure cluster")
        compute_config.add_argument("--compute-tenant-id",
                                    dest="compute_tenant_id",
                                    default=None,
                                    help="tenant id for azure cluster")
        compute_config.add_argument("--compute-subscription-id",
                                    dest="compute_subscription_id",
                                    default=None,
                                    help="Subscription id for azure cluster")

        # location settings parser
        location_group = argparser.add_argument_group("location config settings")
        location_group.add_argument("--location",
                                    dest="location",
                                    default=None,
                                    help="location for azure cluster")

        # network settings parser
        network_config_group = argparser.add_argument_group("network config settings")
        network_config_group.add_argument("--vnet-name",
                                          dest="vnet_name",
                                          help="vnet name for azure", )
        network_config_group.add_argument("--subnet-name",
                                          dest="subnet_name",
                                          help="subnet name for azure")
        network_config_group.add_argument("--vnet-resource-group-name",
                                          dest="vnet_resource_group_name",
                                          help="vnet resource group name for azure")
        network_config_group.add_argument("--master-static-nic-name",
                                          dest="master_static_nic_name",
                                          help="name of NIC to be attached to master node")
        network_config_group.add_argument("--master-static-public-ip-name",
                                          dest="master_static_public_ip_name",
                                          help="name of public IP to be attached to master node")
        network_config_group.add_argument("--resource-group-name",
                                          dest="resource_group_name",
                                          help="resource group for cluster")
        # storage config settings parser
        storage_config = argparser.add_argument_group("storage config settings")
        storage_config.add_argument("--storage-access-key",
                                    dest="storage_access_key",
                                    default=None,
                                    help="storage access key for azure cluster")
        storage_config.add_argument("--storage-account-name",
                                    dest="storage_account_name",
                                    default=None,
                                    help="storage account name for azure cluster")
        storage_config.add_argument("--disk-storage-account-name",
                                    dest="disk_storage_account_name",
                                    default=None,
                                    help="disk storage account name for azure cluster")
        storage_config.add_argument("--disk-storage-account-resource-group-name",
                                    dest="disk_storage_account_resource_group_name",
                                    default=None,
                                    help="disk storage account resource group for azure cluster")


"""
The Accounts module contains the base definition for a Qubole account object
"""
from qds_sdk.resource import SingletonResource
from qds_sdk.qubole import Qubole
import argparse


class AccountCmdLine:
    @staticmethod
    def parsers():
        argparser = argparse.ArgumentParser(
            prog="qds.py account",
            description="Account Creation for Qubole Data Service.")
        subparsers = argparser.add_subparsers(title="account operations")

        # Create
        create = subparsers.add_parser(
            "create", help="Create a new account")
        create.add_argument(
            "--name", dest="name", required=True, help="Account name")
        create.add_argument(
            "--location", dest="defloc", required=True,
            help="Default S3 location for storing logs, result and cached data")
        create.add_argument(
            "--storage-access-key", dest="acc_key", required=True,
            help="AWS Access Key ID for storage")
        create.add_argument(
            "--storage-secret-key", dest="secret", required=True,
            help="AWS Secret Access Key for storage")
        create.add_argument(
            "--compute-access-key", dest="compute_access_key", required=True,
            help="AWS Access Key ID for compute")
        create.add_argument(
            "--compute-secret-key", dest="compute_secret_key", required=True,
            help="AWS Secret Access Key for compute")
        create.add_argument(
            "--aws-region", dest="aws_region", required=True, help="AWS Region",
            choices=["us-east-1", "us-west-2", "ap-northeast-1", "sa-east-1",
                     "eu-west-1", "ap-southeast-1", "us-west-1"])
        create.add_argument(
            "--previous-account-plan", dest="use_previous_account_plan",
            choices=["true", "false"], default="false",
            help="Use previous account plan, default: false")
        create.set_defaults(func=AccountCmdLine.create)

        #branding
        branding = subparsers.add_parser(
            "branding", help="Branding logo and link")
        branding.add_argument("--account-id", dest = "account_id", required=True, help = "Account ID of the Qubole account for which branding has to be done")
        branding.add_argument("--logo-uri", dest = "logo_uri", help = "Publicly accessible logo URI image in jpg/gif/svg/jpeg format.Image size must be less than 100 KB.")
        branding.add_argument("--link-url", dest = "link_url", help = "Specify the documentation URL.")
        branding.add_argument("--link-label", dest = "link_label", help = "Add a label to describe the documentation URL.")
        branding.set_defaults(func=AccountCmdLine.branding)
        return argparser

    @staticmethod
    def run(args):
        parser = AccountCmdLine.parsers()
        parsed = parser.parse_args(args)
        return parsed.func(parsed)

    @staticmethod
    def create(args):
        args.level = "free"
        args.compute_type = "CUSTOMER_MANAGED"
        args.storage_type = "CUSTOMER_MANAGED"
        args.CacheQuotaSizeInGB = "25"
        v = {}
        args = vars(args)
        args.pop("func")
        v['account'] = args
        result = Account.create(**v)
        return result

    @staticmethod
    def branding(args):
        v= {}
        print(args)
        v['account_id'] = args.account_id
        if args.logo_uri is not None:
            v['logo'] = {'logo_uri' : args.logo_uri }

        link = {}
        if args.link_url is not None:
            link['link_url'] = args.link_url
        if args.link_label is not None:
            link['link_label'] = args.link_label

        if bool(link):
            v['link'] = link

        result = Account.branding(**v)
        return result

class Account(SingletonResource):
    credentials_rest_entity_path = "accounts/get_creds"
    rest_entity_path = "account"

    @classmethod
    def create(cls, **kwargs):
        conn = Qubole.agent()
        return cls(conn.post(cls.rest_entity_path, data=kwargs))

    @classmethod
    def branding(cls, **kwargs):
        conn = Qubole.agent()
        url_path = "accounts/branding"
        return  cls(conn.put(url_path, data=kwargs))



import json

class CmdLine:
    @staticmethod
    def filter_fields(resource, fields):
        filtered = {}
        for field in fields:
            filtered[field] = resource[field]
        return filtered

    @staticmethod
    def create(cls, args):
        with open(args.data) as f:
            spec = json.load(f)
        schedule = cls(spec)
        return json.dumps(resource.attributes, sort_keys=True, indent=4)

    @staticmethod
    def list(cls, args):
        resource_list = cls.list(args.page, args.per_page)
        if args.fields:
            for s in resource_list:
                s.attributes = CmdLine.filter_fields(s.attributes, args.fields)
        return json.dumps(resource_list, default=lambda o: o.attributes, sort_keys=True, indent=4)

    @staticmethod
    def view(cls, args):
        cls_instance = cls.find(args.id)
        if args.fields:
            cls_instance.attributes = CmdLine.filter_fields(cls_instance.attributes, args.fields)
        return json.dumps(cls_instance.attributes, sort_keys=True, indent=4)

    @staticmethod
    def delete(cls, args):
        return json.dumps(cls.delete(args.id), sort_keys=True, indent=4)



from qds_sdk.qubole import Qubole
from qds_sdk.resource import Resource
from qds_sdk.cloud.cloud import Cloud
from qds_sdk.engine import Engine
from qds_sdk.cluster_info_v22 import ClusterInfoV22
from qds_sdk import util
import argparse
import json


def str2bool(v):
    return v.lower() in ("yes", "true", "t", "1")

class ClusterCmdLine:

    @staticmethod
    def parsers(action):
        argparser = argparse.ArgumentParser(
            prog="qds.py cluster",
            description="Cluster Operations for Qubole Data Service.")
        subparsers = argparser.add_subparsers(title="Cluster operations")
        if Qubole.version is not None:
            ClusterV2.api_version = Qubole.version
        if action == "create":
            create = subparsers.add_parser("create", help="Create a new cluster")
            ClusterCmdLine.create_update_clone_parser(create, action="create")
            create.set_defaults(func=ClusterV2.create)

        if action == "update":
            update = subparsers.add_parser("update", help="Update the settings of an existing cluster")
            ClusterCmdLine.create_update_clone_parser(update, action="update")
            update.set_defaults(func=ClusterV2.update)

        if action == "clone":
            clone = subparsers.add_parser("clone", help="Clone a cluster from an existing one")
            ClusterCmdLine.create_update_clone_parser(clone, action="clone")
            clone.set_defaults(func=ClusterV2.clone)

        if action == "list":
            li = subparsers.add_parser("list", help="list clusters from existing clusters depending upon state")
            ClusterCmdLine.list_parser(li, action="list")
            li.set_defaults(func=ClusterV2.list)
        return argparser

    @staticmethod
    def list_parser(subparser, action=None, ):

        # cluster info parser
        cluster_info_cls = ClusterInfoFactory.get_cluster_info_cls()
        cluster_info_cls.list_info_parser(subparser, action)

    @staticmethod
    def create_update_clone_parser(subparser, action=None):
        # cloud config parser
        cloud = Qubole.get_cloud()
        cloud.create_parser(subparser)

        # cluster info parser
        cluster_info_cls = ClusterInfoFactory.get_cluster_info_cls()
        cluster_info_cls.cluster_info_parser(subparser, action)

        # engine config parser
        Engine.engine_parser(subparser)

    @staticmethod
    def run(args):
        parser = ClusterCmdLine.parsers(args[0])
        arguments = parser.parse_args(args)
        if args[0] in ["create", "clone", "update"]:
            ClusterCmdLine.get_cluster_create_clone_update(arguments, args[0])
        else:
            return arguments.func(arguments.label, arguments.cluster_id, arguments.state,
                                  arguments.page, arguments.per_page)

    @staticmethod
    def get_cluster_create_clone_update(arguments, action):

        # This will set cluster info and monitoring settings
        cluster_info_cls = ClusterInfoFactory.get_cluster_info_cls()
        cluster_info = cluster_info_cls(arguments.label)
        cluster_info.set_cluster_info_from_arguments(arguments)

        #  This will set cloud config settings
        cloud_config = Qubole.get_cloud()
        cloud_config.set_cloud_config_from_arguments(arguments)

        # This will set engine settings
        engine_config = Engine(flavour=arguments.flavour)
        engine_config.set_engine_config_settings(arguments)
        cluster_request = ClusterCmdLine.get_cluster_request_parameters(cluster_info, cloud_config, engine_config)

        action = action
        if action == "create":
            return arguments.func(cluster_request)
        else:
            return arguments.func(arguments.cluster_id_label, cluster_request)

    @staticmethod
    def get_cluster_request_parameters(cluster_info, cloud_config, engine_config):
        '''
        Use this to return final minimal request from cluster_info, cloud_config or engine_config objects
        Alternatively call util._make_minimal if only one object needs to be implemented
        '''

        cluster_request = {}
        cloud_config = util._make_minimal(cloud_config.__dict__)
        if bool(cloud_config): cluster_request['cloud_config'] = cloud_config

        engine_config = util._make_minimal(engine_config.__dict__)
        if bool(engine_config): cluster_request['engine_config'] = engine_config

        cluster_request.update(util._make_minimal(cluster_info.__dict__))
        return cluster_request

class ClusterInfoFactory:

    @staticmethod
    def get_cluster_info_cls(api_version=None):
        if api_version is None:
            api_version = Qubole.version
        if api_version == "v2":
            return ClusterInfoV2
        elif api_version == "v2.2":
            return ClusterInfoV22
        else:
            return ClusterInfoV2

class ClusterInfoV2(object):
    """
    qds_sdk.ClusterInfoV2 is the class which stores information about a cluster_info.
    You can use objects of this class to create/update/clone a cluster.
    """

    def __init__(self, label):
        """
        Args:
        `label`: A list of labels that identify the cluster. At least one label
            must be provided when creating a cluster.
        """
        self.cluster_info = {}
        self.cluster_info['label'] = label
        self.monitoring = {}
        self.internal = {}  # right now not supported

    def set_cluster_info_from_arguments(self, arguments):
        customer_ssh_key = util._read_file(arguments.customer_ssh_key_file)
        self.set_cluster_info(disallow_cluster_termination=arguments.disallow_cluster_termination,
                              enable_ganglia_monitoring=arguments.enable_ganglia_monitoring,
                              datadog_api_token=arguments.datadog_api_token,
                              datadog_app_token=arguments.datadog_app_token,
                              node_bootstrap=arguments.node_bootstrap_file,
                              master_instance_type=arguments.master_instance_type,
                              slave_instance_type=arguments.slave_instance_type,
                              min_nodes=arguments.initial_nodes,
                              max_nodes=arguments.max_nodes,
                              slave_request_type=arguments.slave_request_type,
                              fallback_to_ondemand=arguments.fallback_to_ondemand,
                              node_base_cooldown_period=arguments.node_base_cooldown_period,
                              node_spot_cooldown_period=arguments.node_spot_cooldown_period,
                              custom_tags=arguments.custom_tags,
                              heterogeneous_config=arguments.heterogeneous_config,
                              maximum_bid_price_percentage=arguments.maximum_bid_price_percentage,
                              timeout_for_request=arguments.timeout_for_request,
                              maximum_spot_instance_percentage=arguments.maximum_spot_instance_percentage,
                              stable_maximum_bid_price_percentage=arguments.stable_maximum_bid_price_percentage,
                              stable_timeout_for_request=arguments.stable_timeout_for_request,
                              stable_spot_fallback=arguments.stable_spot_fallback,
                              spot_block_duration=arguments.spot_block_duration,
                              idle_cluster_timeout=arguments.idle_cluster_timeout,
                              disk_count=arguments.count,
                              disk_type=arguments.disk_type,
                              disk_size=arguments.size,
                              root_disk_size=arguments.root_disk_size,
                              upscaling_config=arguments.upscaling_config,
                              enable_encryption=arguments.encrypted_ephemerals,
                              customer_ssh_key=customer_ssh_key,
                              image_uri_overrides=arguments.image_uri_overrides,
                              env_name=arguments.env_name,
                              python_version=arguments.python_version,
                              r_version=arguments.r_version,
                              disable_cluster_pause=arguments.disable_cluster_pause,
                              paused_cluster_timeout_mins=arguments.paused_cluster_timeout_mins,
                              disable_autoscale_node_pause=arguments.disable_autoscale_node_pause,
                              paused_autoscale_node_timeout_mins=arguments.paused_autoscale_node_timeout_mins,
                              parent_cluster_id=arguments.parent_cluster_id,
                              parent_cluster_label=arguments.parent_cluster_label,
                              image_version=arguments.image_version)

    def set_cluster_info(self,
                         disallow_cluster_termination=None,
                         enable_ganglia_monitoring=None,
                         datadog_api_token=None,
                         datadog_app_token=None,
                         node_bootstrap=None,
                         master_instance_type=None,
                         slave_instance_type=None,
                         min_nodes=None,
                         max_nodes=None,
                         slave_request_type=None,
                         fallback_to_ondemand=None,
                         node_base_cooldown_period=None,
                         node_spot_cooldown_period=None,
                         custom_tags=None,
                         heterogeneous_config=None,
                         maximum_bid_price_percentage=None,
                         timeout_for_request=None,
                         maximum_spot_instance_percentage=None,
                         stable_maximum_bid_price_percentage=None,
                         stable_timeout_for_request=None,
                         stable_spot_fallback=None,
                         spot_block_duration=None,
                         idle_cluster_timeout=None,
                         disk_count=None,
                         disk_type=None,
                         disk_size=None,
                         root_disk_size=None,
                         upscaling_config=None,
                         enable_encryption=None,
                         customer_ssh_key=None,
                         cluster_name=None,
                         force_tunnel=None,
                         image_uri_overrides=None,
                         env_name=None,
                         python_version=None,
                         r_version=None,
                         disable_cluster_pause=None,
                         paused_cluster_timeout_mins=None,
                         disable_autoscale_node_pause=None,
                         paused_autoscale_node_timeout_mins=None,
                         parent_cluster_id=None,
                         parent_cluster_label=None,
                         image_version=None):
        """
        Args:

                `disallow_cluster_termination`: Set this to True if you don't want
                    qubole to auto-terminate idle clusters. Use this option with
                    extreme caution.

                `enable_ganglia_monitoring`: Set this to True if you want to enable
                    ganglia monitoring for the cluster.

                `node_bootstrap`: name of the node bootstrap file for this
                    cluster. It should be in stored in S3 at
                    <your-default-location>/scripts/hadoop/

                `master_instance_type`: The instance type to use for the Hadoop master
                    node.

                `slave_instance_type`: The instance type to use for the Hadoop slave
                    nodes.

                `min_nodes`: Number of nodes to start the cluster with.

                `max_nodes`: Maximum number of nodes the cluster may be auto-scaled up
                    to.

                `slave_request_type`: Purchasing option for slave instances.
                    Valid values: "ondemand", "hybrid", "spot".

                `fallback_to_ondemand`: Fallback to on-demand nodes if spot nodes could not be
                    obtained. Valid only if slave_request_type is 'spot'.

                `node_base_cooldown_period`: Time for which an on-demand node waits before termination (Unit: minutes)

                `node_spot_cooldown_period`: Time for which a spot node waits before termination (Unit: minutes)

                `maximum_bid_price_percentage`: ( Valid only when `slave_request_type`
                    is hybrid or spot.) Maximum value to bid for spot
                    instances, expressed as a percentage of the base price
                    for the slave node instance type.

                `timeout_for_request`: Timeout for a spot instance request (Unit:
                    minutes)

                `maximum_spot_instance_percentage`: Maximum percentage of instances
                    that may be purchased from the AWS Spot market. Valid only when
                    slave_request_type is "hybrid".

                `stable_maximum_bid_price_percentage`: Maximum value to bid for stable node spot
                    instances, expressed as a percentage of the base price
                    (applies to both master and slave nodes).

                `stable_timeout_for_request`: Timeout for a stable node spot instance request (Unit:
                    minutes)

                `stable_spot_fallback`: Whether to fallback to on-demand instances for
                    stable nodes if spot instances are not available

                `spot_block_duration`: Time for which the spot block instance is provisioned (Unit:
                    minutes)

                `disk_count`: Number of EBS volumes to attach
                    to each instance of the cluster.

                `disk_type`: Type of the EBS volume. Valid
                    values are 'standard' (magnetic) and 'ssd'.

                `disk_size`: Size of each EBS volume, in GB.

                `root_disk_size`: Size of root volume, in GB.

                `enable_encryption`: Encrypt the ephemeral drives on the instance.

                `customer_ssh_key`: SSH key to use to login to the instances.

                `idle_cluster_timeout`: The buffer time (range in 0-6 hrs) after a cluster goes idle
                    and gets terminated,  given cluster auto termination is on and no cluster specific
                    timeout has been set (default is 2 hrs)

                `heterogeneous_config` : Configuring heterogeneous nodes in Hadoop 2 and Spark clusters.
                    It implies that slave nodes can be of different instance types

                `custom_tags` : Custom tags to be set on all instances
                    of the cluster. Specified as JSON object (key-value pairs)

                `datadog_api_token` : Specify the Datadog API token to use the Datadog monitoring service

                `datadog_app_token` : Specify the Datadog APP token to use the Datadog monitoring service

                `image_uri_overrides` : Override the image name provided

                `env_name`: Name of python and R environment. (For Spark clusters)

                `python_version`: Version of Python for environment. (For Spark clusters)

                `r_version`: Version of R for environment. (For Spark clusters)

                `disable_cluster_pause`: Disable cluster pause

                `paused_cluster_timeout_mins`: Paused cluster timeout in mins

                `disable_autoscale_node_pause`: Disable autoscale node pause

                `paused_autoscale_node_timeout_mins`: Paused autoscale node timeout in mins

                `parent_cluster_id`: parent cluster id for HS2 cluster

                `image_version`: cluster image version

        Doc: For getting details about arguments
        http://docs.qubole.com/en/latest/rest-api/cluster_api/create-new-cluster.html#parameters

        """
        self.cluster_info['master_instance_type'] = master_instance_type
        self.cluster_info['slave_instance_type'] = slave_instance_type
        self.cluster_info['min_nodes'] = min_nodes
        self.cluster_info['max_nodes'] = max_nodes
        self.cluster_info['cluster_name'] = cluster_name
        self.cluster_info['node_bootstrap'] = node_bootstrap
        self.cluster_info['disallow_cluster_termination'] = disallow_cluster_termination
        self.cluster_info['force_tunnel'] = force_tunnel
        self.cluster_info['fallback_to_ondemand'] = fallback_to_ondemand
        self.cluster_info['node_base_cooldown_period'] = node_base_cooldown_period
        self.cluster_info['node_volatile_cooldown_period'] = node_spot_cooldown_period
        self.cluster_info['customer_ssh_key'] = customer_ssh_key
        if custom_tags and custom_tags.strip():
            try:
                self.cluster_info['custom_tags'] = json.loads(custom_tags.strip())
            except Exception as e:
                raise Exception("Invalid JSON string for custom ec2 tags: %s" % e.message)

        self.cluster_info['heterogeneous_config'] = heterogeneous_config
        self.cluster_info['slave_request_type'] = slave_request_type
        self.cluster_info['idle_cluster_timeout'] = idle_cluster_timeout
        self.cluster_info['spot_settings'] = {}

        self.cluster_info['rootdisk'] = {}
        self.cluster_info['rootdisk']['size'] = root_disk_size
        self.cluster_info['parent_cluster_id'] = parent_cluster_id
        self.cluster_info['parent_cluster_label'] = parent_cluster_label
        self.cluster_info['cluster_image_version'] = image_version

        self.set_spot_instance_settings(maximum_bid_price_percentage, timeout_for_request,
                                        maximum_spot_instance_percentage)
        self.set_stable_spot_bid_settings(stable_maximum_bid_price_percentage, stable_timeout_for_request,
                                          stable_spot_fallback)
        self.set_spot_block_settings(spot_block_duration)
        self.set_data_disk(disk_size, disk_count, disk_type, upscaling_config, enable_encryption)
        self.set_monitoring(enable_ganglia_monitoring, datadog_api_token, datadog_app_token)
        self.set_internal(image_uri_overrides)
        self.set_env_settings(env_name, python_version, r_version)
        self.set_start_stop_settings(disable_cluster_pause, paused_cluster_timeout_mins,
                                     disable_autoscale_node_pause, paused_autoscale_node_timeout_mins)

    def set_datadog_setting(self,
                            datadog_api_token=None,
                            datadog_app_token=None):
        self.monitoring['datadog'] = {}
        self.monitoring['datadog']['datadog_api_token'] = datadog_api_token
        self.monitoring['datadog']['datadog_app_token'] = datadog_app_token

    def set_monitoring(self,
                       enable_ganglia_monitoring=None,
                       datadog_api_token=None,
                       datadog_app_token=None):
        self.monitoring['ganglia'] = enable_ganglia_monitoring
        self.set_datadog_setting(datadog_api_token, datadog_app_token)

    def set_spot_instance_settings(self,
                                   maximum_bid_price_percentage=None,
                                   timeout_for_request=None,
                                   maximum_spot_instance_percentage=None):
        self.cluster_info['spot_settings']['spot_instance_settings'] = {}
        self.cluster_info['spot_settings']['spot_instance_settings']['maximum_bid_price_percentage'] = \
            maximum_bid_price_percentage
        self.cluster_info['spot_settings']['spot_instance_settings']['timeout_for_request'] = timeout_for_request
        self.cluster_info['spot_settings']['spot_instance_settings']['maximum_spot_instance_percentage'] = \
            maximum_spot_instance_percentage

    def set_stable_spot_bid_settings(self,
                                     stable_maximum_bid_price_percentage=None,
                                     stable_timeout_for_request=None,
                                     stable_spot_fallback=None):
        self.cluster_info['spot_settings']['stable_spot_bid_settings'] = {}
        self.cluster_info['spot_settings']['stable_spot_bid_settings']['maximum_bid_price_percentage'] = \
            stable_maximum_bid_price_percentage
        self.cluster_info['spot_settings']['stable_spot_bid_settings']['timeout_for_request'] = \
            stable_timeout_for_request
        self.cluster_info['spot_settings']['stable_spot_bid_settings']['stable_spot_fallback'] = \
            stable_spot_fallback

    def set_spot_block_settings(self,
                                spot_block_duration=None):
        self.cluster_info['spot_settings']['spot_block_settings'] = {}
        self.cluster_info['spot_settings']['spot_block_settings']['duration'] = spot_block_duration

    def set_data_disk(self,
                      disk_size=None,
                      disk_count=None,
                      disk_type=None,
                      upscaling_config=None,
                      enable_encryption=None):
        self.cluster_info['datadisk'] = {}
        self.cluster_info['datadisk']['size'] = disk_size
        self.cluster_info['datadisk']['count'] = disk_count
        self.cluster_info['datadisk']['type'] = disk_type
        self.cluster_info['datadisk']['upscaling_config'] = upscaling_config
        self.cluster_info['datadisk']['encryption'] = enable_encryption

    def set_internal(self, image_uri_overrides=None):
        self.internal['image_uri_overrides'] = image_uri_overrides

    def set_env_settings(self, env_name=None, python_version=None, r_version=None):
        self.cluster_info['env_settings'] = {}
        self.cluster_info['env_settings']['name'] = env_name
        self.cluster_info['env_settings']['python_version'] = python_version
        self.cluster_info['env_settings']['r_version'] = r_version

    def set_start_stop_settings(self,
                                disable_cluster_pause=None,
                                paused_cluster_timeout_mins=None,
                                disable_autoscale_node_pause=None,
                                paused_autoscale_node_timeout_mins=None):
        if disable_cluster_pause is not None:
            disable_cluster_pause = int(disable_cluster_pause)
        self.cluster_info['disable_cluster_pause'] = disable_cluster_pause
        self.cluster_info['paused_cluster_timeout_mins'] = paused_cluster_timeout_mins
        if disable_autoscale_node_pause is not None:
            disable_autoscale_node_pause = int(disable_autoscale_node_pause)
        self.cluster_info['disable_autoscale_node_pause'] = disable_autoscale_node_pause
        self.cluster_info['paused_autoscale_node_timeout_mins'] = paused_autoscale_node_timeout_mins

    @staticmethod
    def list_info_parser(argparser, action):
        argparser.add_argument("--id", dest="cluster_id",
                               help="show cluster with this id")

        argparser.add_argument("--label", dest="label",
                               help="show cluster with this label")
        argparser.add_argument("--state", dest="state",
                               choices=['invalid', 'up', 'down', 'pending', 'terminating'],
                               help="State of the cluster")
        argparser.add_argument("--page", dest="page",
                               type=int,
                               help="Page number")
        argparser.add_argument("--per-page", dest="per_page",
                               type=int,
                               help="Number of clusters to be retrieved per page")

    @staticmethod
    def cluster_info_parser(argparser, action):
        create_required = False
        label_required = False
        if action == "create":
            create_required = True
        elif action == "update":
            argparser.add_argument("cluster_id_label",
                                   help="id/label of the cluster to update")
        elif action == "clone":
            argparser.add_argument("cluster_id_label",
                                   help="id/label of the cluster to update")
            label_required = True

        argparser.add_argument("--label", dest="label",
                               nargs="+", required=(create_required or label_required),
                               help="list of labels for the cluster" +
                                    " (atleast one label is required)")
        cluster_info = argparser.add_argument_group("cluster_info")
        cluster_info.add_argument("--master-instance-type",
                                  dest="master_instance_type",
                                  help="instance type to use for the hadoop" +
                                       " master node")
        cluster_info.add_argument("--slave-instance-type",
                                  dest="slave_instance_type",
                                  help="instance type to use for the hadoop" +
                                       " slave nodes")
        cluster_info.add_argument("--min-nodes",
                                  dest="initial_nodes",
                                  type=int,
                                  help="number of nodes to start the" +
                                       " cluster with", )
        cluster_info.add_argument("--max-nodes",
                                  dest="max_nodes",
                                  type=int,
                                  help="maximum number of nodes the cluster" +
                                       " may be auto-scaled up to")
        cluster_info.add_argument("--idle-cluster-timeout",
                                  dest="idle_cluster_timeout",
                                  help="cluster termination timeout for idle cluster")
        cluster_info.add_argument("--node-bootstrap-file",
                                  dest="node_bootstrap_file",
                                  help="""name of the node bootstrap file for this cluster. It
                                   should be in stored in S3 at
                                   <account-default-location>/scripts/hadoop/NODE_BOOTSTRAP_FILE
                                   """, )
        cluster_info.add_argument("--root-disk-size",
                                  dest="root_disk_size",
                                  type=int,
                                  help="size of the root volume in GB")
        cluster_info.add_argument("--parent-cluster-id",
                                  dest="parent_cluster_id",
                                  type=int,
                                  help="Id of the parent cluster this hs2 cluster is attached to")
        cluster_info.add_argument("--parent-cluster-label",
                                  dest="parent_cluster_label",
                                  help="Label of the parent cluster this hs2 cluster is attached to")
        cluster_info.add_argument("--image-version",
                                  dest="image_version",
                                  help="cluster image version")
        termination = cluster_info.add_mutually_exclusive_group()
        termination.add_argument("--disallow-cluster-termination",
                                 dest="disallow_cluster_termination",
                                 action="store_true",
                                 default=None,
                                 help="don't auto-terminate idle clusters," +
                                      " use this with extreme caution", )
        termination.add_argument("--allow-cluster-termination",
                                 dest="disallow_cluster_termination",
                                 action="store_false",
                                 default=None,
                                 help="auto-terminate idle clusters,")
        fallback_to_ondemand_group = cluster_info.add_mutually_exclusive_group()
        fallback_to_ondemand_group.add_argument("--fallback-to-ondemand",
                                                dest="fallback_to_ondemand",
                                                action="store_true",
                                                default=None,
                                                help="Fallback to on-demand nodes if spot nodes" +
                                                     " could not be obtained. Valid only if slave_request_type is spot", )
        fallback_to_ondemand_group.add_argument("--no-fallback-to-ondemand",
                                                dest="fallback_to_ondemand",
                                                action="store_false",
                                                default=None,
                                                help="Dont Fallback to on-demand nodes if spot nodes" +
                                                     " could not be obtained. Valid only if slave_request_type is spot", )
        node_cooldown_period_group = argparser.add_argument_group("node cooldown period settings")
        node_cooldown_period_group.add_argument("--node-base-cooldown-period",
                                                dest="node_base_cooldown_period",
                                                type=int,
                                                help="Cooldown period for on-demand nodes" +
                                                     " unit: minutes")
        node_cooldown_period_group.add_argument("--node-spot-cooldown-period",
                                                dest="node_spot_cooldown_period",
                                                type=int,
                                                help="Cooldown period for spot nodes" +
                                                     " unit: minutes")
        cluster_info.add_argument("--customer-ssh-key",
                                  dest="customer_ssh_key_file",
                                  help="location for ssh key to use to" +
                                       " login to the instance")
        cluster_info.add_argument("--custom-tags",
                                  dest="custom_tags",
                                  help="""Custom tags to be set on all instances
                                                 of the cluster. Specified as JSON object (key-value pairs)
                                                 e.g. --custom-ec2-tags '{"key1":"value1", "key2":"value2"}'
                                                 """, )

        # datadisk settings
        datadisk_group = argparser.add_argument_group("data disk settings")
        datadisk_group.add_argument("--count",
                                    dest="count",
                                    type=int,
                                    help="Number of EBS volumes to attach to" +
                                         " each instance of the cluster", )
        datadisk_group.add_argument("--disk-type",
                                    dest="disk_type",
                                    choices=["standard", "gp2"],
                                    help="Type of the  volume attached to the instances. Valid values are " +
                                         "'standard' (magnetic) and 'gp2' (ssd).")
        datadisk_group.add_argument("--size",
                                    dest="size",
                                    type=int,
                                    help="Size of each EBS volume, in GB", )
        datadisk_group.add_argument("--upscaling-config",
                                    dest="upscaling_config",
                                    help="Upscaling config to be attached with the instances.", )
        ephemerals = datadisk_group.add_mutually_exclusive_group()
        ephemerals.add_argument("--encrypted-ephemerals",
                                dest="encrypted_ephemerals",
                                action="store_true",
                                default=None,
                                help="encrypt the ephemeral drives on" +
                                     " the instance", )
        ephemerals.add_argument("--no-encrypted-ephemerals",
                                dest="encrypted_ephemerals",
                                action="store_false",
                                default=None,
                                help="don't encrypt the ephemeral drives on" +
                                     " the instance", )

        cluster_info.add_argument("--heterogeneous-config",
                                  dest="heterogeneous_config",
                                  help="heterogeneous config for the cluster")

        cluster_info.add_argument("--slave-request-type",
                                  dest="slave_request_type",
                                  choices=["ondemand", "spot", "hybrid", "spotblock"],
                                  help="purchasing option for slave instaces", )

        # spot settings
        spot_instance_group = argparser.add_argument_group("spot instance settings" +
                                                           " (valid only when slave-request-type is hybrid or spot)")
        spot_instance_group.add_argument("--maximum-bid-price-percentage",
                                         dest="maximum_bid_price_percentage",
                                         type=float,
                                         help="maximum value to bid for spot instances" +
                                              " expressed as a percentage of the base" +
                                              " price for the slave node instance type", )
        spot_instance_group.add_argument("--timeout-for-spot-request",
                                         dest="timeout_for_request",
                                         type=int,
                                         help="timeout for a spot instance request" +
                                              " unit: minutes")
        spot_instance_group.add_argument("--maximum-spot-instance-percentage",
                                         dest="maximum_spot_instance_percentage",
                                         type=int,
                                         help="maximum percentage of instances that may" +
                                              " be purchased from the aws spot market," +
                                              " valid only when slave-request-type" +
                                              " is 'hybrid'", )

        stable_spot_group = argparser.add_argument_group("stable spot instance settings")
        stable_spot_group.add_argument("--stable-maximum-bid-price-percentage",
                                       dest="stable_maximum_bid_price_percentage",
                                       type=float,
                                       help="maximum value to bid for stable node spot instances" +
                                            " expressed as a percentage of the base" +
                                            " price for the master and slave node instance types", )
        stable_spot_group.add_argument("--stable-timeout-for-spot-request",
                                       dest="stable_timeout_for_request",
                                       type=int,
                                       help="timeout for a stable node spot instance request" +
                                            " unit: minutes")
        stable_spot_group.add_argument("--stable-allow-fallback",
                                       dest="stable_spot_fallback", default=None,
                                       type=str2bool,
                                       help="whether to fallback to on-demand instances for stable nodes" +
                                            " if spot instances aren't available")

        spot_block_group = argparser.add_argument_group("spot block settings")
        spot_block_group.add_argument("--spot-block-duration",
                                      dest="spot_block_duration",
                                      type=int,
                                      help="spot block duration" +
                                           " unit: minutes")

        # monitoring settings
        monitoring_group = argparser.add_argument_group("monitoring settings")
        ganglia = monitoring_group.add_mutually_exclusive_group()
        ganglia.add_argument("--enable-ganglia-monitoring",
                             dest="enable_ganglia_monitoring",
                             action="store_true",
                             default=None,
                             help="enable ganglia monitoring for the" +
                                  " cluster", )
        ganglia.add_argument("--disable-ganglia-monitoring",
                             dest="enable_ganglia_monitoring",
                             action="store_false",
                             default=None,
                             help="disable ganglia monitoring for the" +
                                  " cluster", )

        datadog_group = argparser.add_argument_group("datadog settings")
        datadog_group.add_argument("--datadog-api-token",
                                   dest="datadog_api_token",
                                   default=None,
                                   help="fernet key for airflow cluster", )
        datadog_group.add_argument("--datadog-app-token",
                                   dest="datadog_app_token",
                                   default=None,
                                   help="overrides for airflow cluster", )

        internal_group = argparser.add_argument_group("internal settings")
        internal_group.add_argument("--image-overrides",
                                    dest="image_uri_overrides",
                                    default=None,
                                    help="overrides for image", )

        env_group = argparser.add_argument_group("environment settings")
        env_group.add_argument("--env-name",
                               dest="env_name",
                               default=None,
                               help="name of Python and R environment")
        env_group.add_argument("--python-version",
                               dest="python_version",
                               default=None,
                               help="version of Python in environment")
        env_group.add_argument("--r-version",
                               dest="r_version",
                               default=None,
                               help="version of R in environment")

        start_stop_group = argparser.add_argument_group("start stop settings")
        start_stop_group.add_argument("--disable-cluster-pause",
                                      dest="disable_cluster_pause",
                                      action='store_true',
                                      default=None,
                                      help="disable cluster pause")
        start_stop_group.add_argument("--no-disable-cluster-pause",
                                      dest="disable_cluster_pause",
                                      action='store_false',
                                      default=None,
                                      help="disable cluster pause")
        start_stop_group.add_argument("--paused-cluster-timeout",
                                      dest="paused_cluster_timeout_mins",
                                      default=None,
                                      type=int,
                                      help="paused cluster timeout in min")
        start_stop_group.add_argument("--disable-autoscale-node-pause",
                                      dest="disable_autoscale_node_pause",
                                      action='store_true',
                                      default=None,
                                      help="disable autoscale node pause")
        start_stop_group.add_argument("--no-disable-autoscale-node-pause",
                                      dest="disable_autoscale_node_pause",
                                      action='store_false',
                                      default=None,
                                      help="disable autoscale node pause")
        start_stop_group.add_argument("--paused-autoscale-node-timeout",
                                      dest="paused_autoscale_node_timeout_mins",
                                      default=None,
                                      type=int,
                                      help="paused autoscale node timeout in min")


class ClusterV2(Resource):
    rest_entity_path = "clusters"
    api_version = "v2"

    @classmethod
    def create(cls, cluster_info):
        """
        Create a new cluster using information provided in `cluster_info`.
        """

        conn = Qubole.agent(version=cls.api_version)
        return conn.post(cls.rest_entity_path, data=cluster_info)

    @classmethod
    def update(cls, cluster_id_label, cluster_info):
        """
        Update the cluster with id/label `cluster_id_label` using information provided in
        `cluster_info`.
        """
        conn = Qubole.agent(version=cls.api_version)
        return conn.put(cls.element_path(cluster_id_label), data=cluster_info)

    @classmethod
    def clone(cls, cluster_id_label, cluster_info):
        """
        Update the cluster with id/label `cluster_id_label` using information provided in
        `cluster_info`.
        """
        conn = Qubole.agent(version=cls.api_version)
        return conn.post(cls.element_path(cluster_id_label) + '/clone', data=cluster_info)

    @classmethod
    def list(cls, label=None, cluster_id=None, state=None, page=None, per_page=None):
        """
        List existing clusters present in your account.

        Kwargs:
            `state`: list only those clusters which are in this state
            `page`: page number
            `per_page`: number of clusters to be retrieved per page

        Returns:
            List of clusters satisfying the given criteria
        """
        if cluster_id is not None:
            return cls.show(cluster_id)
        if label is not None:
            return cls.show(label)
        params = {}
        if page:
            params['page'] = page
        if per_page:
            params['per_page'] = per_page
        params = None if not params else params
        conn = Qubole.agent(version=cls.api_version)
        cluster_list = conn.get(cls.rest_entity_path)
        if state is None:
            # return the complete list since state is None
            return conn.get(cls.rest_entity_path, params=params)
        # filter clusters based on state
        result = []
        if 'clusters' in cluster_list:
            for cluster in cluster_list['clusters']:
                if state.lower() == cluster['state'].lower():
                    result.append(cluster)
        return result

    @classmethod
    def show(cls, cluster_id_label):
        """
        Show information about the cluster with id/label `cluster_id_label`.
        """
        conn = Qubole.agent(version=cls.api_version)
        return conn.get(cls.element_path(cluster_id_label))



from qds_sdk.cloud.cloud import Cloud
class AwsCloud(Cloud):
    '''
    qds_sdk.cloud.AwsCloud is the class which stores information about aws cloud config settings.
    The objects of this class can be used to set aws cloud_config settings while create/update/clone a cluster.
    '''

    def __init__(self):
        self.compute_config = {}
        self.location = {}
        self.network_config = {}
        self.storage_config = {}

    def set_cloud_config(self,
                         compute_access_key=None,
                         compute_secret_key=None,
                         use_account_compute_creds=None,
                         aws_region=None,
                         aws_availability_zone=None,
                         role_instance_profile=None,
                         vpc_id=None,
                         subnet_id=None,
                         persistent_security_groups=None,
                         bastion_node_public_dns=None,
                         master_elastic_ip=None):
        '''

        Args:
            compute_access_key: The access key for customer's aws account. This
                is required for creating the cluster.

            compute_secret_key: The secret access key for customer's aws
                account. This is required for creating the cluster.

            use_account_compute_creds: Set it to true to use the account's compute
                credentials for all clusters of the account.The default value is false

            aws_region: The AWS region in which the cluster is created. The default value is, us-east-1.
                Valid values are, us-east-1, us-west-1, us-west-2, eu-west-1, sa-east1, ap-southeast-1,
                and ap-northeast-1.
                Doc: http://docs.qubole.com/en/latest/rest-api/cluster_api/create-new-cluster.html#ec2-settings

            aws_availability_zone: The preferred availability zone in which the cluster must be created. The default value is Any.

            role_instance_profile: IAM Role instance profile to attach on cluster

            vpc_id: The ID of the vpc in which the cluster is created.
                In this vpc, the enableDnsHostnames parameter must be set to true.

            subnet_id: The ID of the subnet in which the cluster is created. This subnet must belong to the
                above VPC and it can be a public/private subnet

            persistent_security_groups: security group to associate with each node of the cluster.
                Typically used to provide access to external hosts

            bastion_node_public_dns: Specify the Bastion host public DNS name if private subnet is provided.
                Do not specify this value for a public subnet.

            master_elastic_ip: It is the Elastic IP address for attaching to the cluster master

        '''

        self.set_compute_config(use_account_compute_creds, compute_access_key,
                                compute_secret_key, role_instance_profile)
        self.set_location(aws_region, aws_availability_zone)
        self.set_network_config(bastion_node_public_dns, persistent_security_groups,
                                master_elastic_ip, vpc_id, subnet_id)

    def set_compute_config(self,
                           use_account_compute_creds=None,
                           compute_access_key=None,
                           compute_secret_key=None,
                           role_instance_profile=None):
        self.compute_config['use_account_compute_creds'] = use_account_compute_creds
        self.compute_config['compute_access_key'] = compute_access_key
        self.compute_config['compute_secret_key'] = compute_secret_key
        self.compute_config['role_instance_profile'] = role_instance_profile

    def set_location(self,
                     aws_region=None,
                     aws_availability_zone=None):
        self.location['aws_region'] = aws_region
        self.location['aws_availability_zone'] = aws_availability_zone

    def set_network_config(self,
                           bastion_node_public_dns=None,
                           persistent_security_groups=None,
                           master_elastic_ip=None,
                           vpc_id=None,
                           subnet_id=None):
        self.network_config['bastion_node_public_dns'] = bastion_node_public_dns
        self.network_config['persistent_security_groups'] = persistent_security_groups
        self.network_config['master_elastic_ip'] = master_elastic_ip
        self.network_config['vpc_id'] = vpc_id
        self.network_config['subnet_id'] = subnet_id

    def set_cloud_config_from_arguments(self, arguments):
        self.set_cloud_config(compute_access_key=arguments.compute_access_key,
                              compute_secret_key=arguments.compute_secret_key,
                              use_account_compute_creds=arguments.use_account_compute_creds,
                              aws_region=arguments.aws_region,
                              aws_availability_zone=arguments.aws_availability_zone,
                              role_instance_profile=arguments.role_instance_profile,
                              vpc_id=arguments.vpc_id,
                              subnet_id=arguments.subnet_id,
                              persistent_security_groups=arguments.persistent_security_groups,
                              bastion_node_public_dns=arguments.bastion_node_public_dns,
                              master_elastic_ip=arguments.master_elastic_ip)

    def create_parser(self, argparser):
        # compute settings parser
        compute_config = argparser.add_argument_group("compute config settings")
        compute_creds = compute_config.add_mutually_exclusive_group()
        compute_creds.add_argument("--enable-account-compute-creds",
                                   dest="use_account_compute_creds",
                                   action="store_true",
                                   default=None,
                                   help="to use account compute credentials")
        compute_creds.add_argument("--disable-account-compute-creds",
                                   dest="use_account_compute_creds",
                                   action="store_false",
                                   default=None,
                                   help="to disable account compute credentials")
        compute_config.add_argument("--compute-access-key",
                                    dest="compute_access_key",
                                    default=None,
                                    help="access key for aws cluster")
        compute_config.add_argument("--compute-secret-key",
                                    dest="compute_secret_key",
                                    default=None,
                                    help="secret key for aws cluster")
        compute_config.add_argument("--role-instance-profile",
                                    dest="role_instance_profile",
                                    help="IAM Role instance profile to attach on cluster", )

        # location settings parser
        location_group = argparser.add_argument_group("location config setttings")
        location_group.add_argument("--aws-region",
                                    dest="aws_region",
                                    choices=["us-east-1", "us-west-2", "ap-northeast-1", "sa-east-1",
                                             "eu-west-1", "ap-southeast-1", "us-west-1"],
                                    help="aws region to create the cluster in", )
        location_group.add_argument("--aws-availability-zone",
                                    dest="aws_availability_zone",
                                    help="availability zone to" +
                                         " create the cluster in", )

        # network settings parser
        network_config_group = argparser.add_argument_group("network config settings")
        network_config_group.add_argument("--vpc-id",
                                          dest="vpc_id",
                                          help="vpc to create the cluster in", )
        network_config_group.add_argument("--subnet-id",
                                          dest="subnet_id",
                                          help="subnet to create the cluster in", )
        network_config_group.add_argument("--bastion-node-public-dns",
                                          dest="bastion_node_public_dns",
                                          help="public dns name of the bastion node. Required only if cluster is in private subnet of a EC2-VPC", )
        network_config_group.add_argument("--persistent-security-groups",
                                          dest="persistent_security_groups",
                                          help="a security group to attach with each" +
                                               " node of the cluster. Typically used" +
                                               " to provide access to external hosts", )
        network_config_group.add_argument("--master-elastic-ip",
                                          dest="master_elastic_ip",
                                          help="master elastic ip for cluster")


import requests
import logging
from qds_sdk.connection import Connection
from qds_sdk.exception import ConfigError

log = logging.getLogger("qds_qubole")

class QuboleAuth(requests.auth.AuthBase):
    def __init__(self, token):
        self.api_token = token

    def __call__(self, r):
        r.headers['X-AUTH-TOKEN'] = self.api_token
        return r


class Qubole:
    """
    Singleton for storing authorization credentials and other
    configuration parameters for QDS.
    """

    MIN_POLL_INTERVAL = 1
    RETRIES_CAP = 5
    MAX_RETRY_DELAY = 10

    _auth = None
    api_token = None
    baseurl = None
    version = None
    poll_interval = None
    skip_ssl_cert_check = None
    cloud_name = None
    cached_agent = None
    cloud = None
    base_retry_delay = None
    max_retries = None

    @classmethod
    def configure(cls, api_token,
                  api_url="https://api.qubole.com/api/", version="v1.2",
                  poll_interval=5, skip_ssl_cert_check=False, cloud_name="AWS",
                  base_retry_delay=10, max_retries=6):
        """
        Set parameters governing interaction with QDS
        Args:
            `api_token`: authorization token for QDS. required
            `api_url`: the base URL for QDS API. configurable for testing only
            `version`: QDS REST api version. Will be used throughout unless overridden in Qubole.agent(..)
            `poll_interval`: interval in secs when polling QDS for events
            `delay` : interval in secs to sleep in between successive retries
            `retries` : maximum number of time to retry an api call in case
                        of retryable exception.
        """

        cls._auth = QuboleAuth(api_token)
        cls.api_token = api_token
        cls.version = version
        cls.baseurl = api_url
        if poll_interval < Qubole.MIN_POLL_INTERVAL:
            log.warn("Poll interval cannot be less than %s seconds. Setting it to %s seconds.\n" % (Qubole.MIN_POLL_INTERVAL, Qubole.MIN_POLL_INTERVAL))
            cls.poll_interval = Qubole.MIN_POLL_INTERVAL
        else:
            cls.poll_interval = poll_interval
        cls.skip_ssl_cert_check = skip_ssl_cert_check
        cls.cloud_name = cloud_name.lower()
        cls.cached_agent = None
        if base_retry_delay > Qubole.MAX_RETRY_DELAY:
            log.warn("Sleep between successive retries cannot be greater than"
                     " %s seconds."
                     " Setting it to"
                     " %s seconds.\n"
                     % (Qubole.MAX_RETRY_DELAY, Qubole.MAX_RETRY_DELAY))
            cls.base_retry_delay = Qubole.MAX_RETRY_DELAY
        else:
            cls.base_retry_delay = base_retry_delay
        if max_retries > Qubole.RETRIES_CAP:
            log.warn("Maximum retries cannot be greater than %s."
                     " Setting it to"
                     " default - %s.\n" % (Qubole.RETRIES_CAP, Qubole.RETRIES_CAP))
            cls.max_retries = Qubole.RETRIES_CAP
        else:
            cls.max_retries = max_retries

    @classmethod
    def agent(cls, version=None):
        """
        Returns:
           a connection object to make REST calls to QDS
           optionally override the `version` of the REST endpoint for advanced
           features available only in the newer version of the API available
           for certain resource end points eg: /v1.3/cluster. When version is
           None we default to v1.2
        """
        reuse_cached_agent = True
        if version:
            log.debug("api version changed to %s" % version)
            cls.rest_url = '/'.join([cls.baseurl.rstrip('/'), version])
            reuse_cached_agent = False
        else:
            cls.rest_url = '/'.join([cls.baseurl.rstrip('/'), cls.version])
        if cls.api_token is None:
            raise ConfigError("No API Token specified - please supply one via Qubole.configure()")

        if not reuse_cached_agent:
            uncached_agent = Connection(cls._auth, cls.rest_url,
                                        cls.skip_ssl_cert_check,
                                        True, cls.max_retries, cls.base_retry_delay)
            return uncached_agent
        if cls.cached_agent is None:
            cls.cached_agent = Connection(cls._auth, cls.rest_url,
                                          cls.skip_ssl_cert_check,
                                          True, cls.max_retries, cls.base_retry_delay)

        return cls.cached_agent

    @classmethod
    def get_cloud(cls, cloud_name=None):
        if cloud_name and cloud_name.lower() not in ["aws", "oracle_bmc", "azure", "oracle_opc", "gcp"]:
            raise Exception("cloud should be 'aws', 'oracle_bmc', 'azure', 'oracle_opc' or 'gcp'")

        if cloud_name:
            return Qubole.get_cloud_object(cloud_name)
        else:
            if cls.cloud is None:
                cls.cloud = cls.get_cloud_object(cls.cloud_name)
            return cls.cloud

    @classmethod
    def get_cloud_object(cls, cloud_name):
        if cloud_name.lower() == "aws":
            import qds_sdk.cloud.aws_cloud
            return qds_sdk.cloud.aws_cloud.AwsCloud()
        elif cloud_name.lower()  == "oracle_bmc":
            import qds_sdk.cloud.oracle_bmc_cloud
            return qds_sdk.cloud.oracle_bmc_cloud.OracleBmcCloud()
        elif cloud_name.lower()  == "azure":
            import qds_sdk.cloud.azure_cloud
            return qds_sdk.cloud.azure_cloud.AzureCloud()
        elif cloud_name.lower()  == "oracle_opc":
            import qds_sdk.cloud.oracle_opc_cloud
            return qds_sdk.cloud.oracle_opc_cloud.OracleOpcCloud()
        elif cloud_name.lower() == "gcp":
            import qds_sdk.cloud.gcp_cloud
            return qds_sdk.cloud.gcp_cloud.GcpCloud()



from qds_sdk.cloud.cloud import Cloud
class OracleOpcCloud(Cloud):
    '''
    qds_sdk.cloud.OracleOpcCloud is the class which stores information about oracle opc cloud config settings.
    The objects of this class can be use to set oracle opc cloud_config settings while create/update/clone a cluster.
    '''

    def __init__(self):
        self.compute_config = {}
        self.network_config = {}
        self.storage_config = {}

    def set_cloud_config(self,
                         username=None,
                         password=None,
                         rest_api_endpoint=None,
                         use_account_compute_creds=None,
                         storage_rest_api_endpoint=None,
                         storage_username=None,
                         storage_password=None,
                         acl=None,
                         ip_network=None,
                         data_disk_count=None,
                         data_disk_size=None):
        '''

        Args:
            username: Username for customers oracle opc account. This
                is required for creating the cluster.

            password: Password for customers oracle opc account. This
                is required for creating the cluster.

            rest_api_endpoint: Rest API endpoint for customers oracle opc cloud.

            use_account_compute_creds: Set it to true to use the accounts compute
                credentials for all clusters of the account.The default value is false

            storage_rest_api_endpoint: Rest API endpoint for storage related operations.

            storage_username: Username for customers oracle opc account. This
                is required for creating the cluster.

            storage_password: Password for customers oracle opc account. This
                is required for creating the cluster.

            acl: acl for oracle opc.

            ip_network: subnet name for oracle opc

        '''

        self.set_compute_config(use_account_compute_creds, username,
                                password, rest_api_endpoint)
        self.set_network_config(acl, ip_network)
        self.set_storage_config(storage_username, storage_password,
                                storage_rest_api_endpoint, data_disk_count, data_disk_size)

    def set_compute_config(self,
                           use_account_compute_creds=None,
                           username=None,
                           password=None,
                           rest_api_endpoint=None):
        self.compute_config['use_account_compute_creds'] = use_account_compute_creds
        self.compute_config['username'] = username
        self.compute_config['password'] = password
        self.compute_config['rest_api_endpoint'] = rest_api_endpoint

    
    def set_network_config(self,
                           acl=None,
                           ip_network=None):
        self.network_config['acl'] = acl
        self.network_config['ip_network'] = ip_network

    def set_storage_config(self,
                           storage_username=None,
                           storage_password=None,
                           storage_rest_api_endpoint=None,
                           data_disk_count=None,
                           data_disk_size=None):
        self.storage_config['storage_username'] = storage_username
        self.storage_config['storage_password'] = storage_password
        self.storage_config['storage_rest_api_endpoint'] = storage_rest_api_endpoint
        self.storage_config['data_disk_count'] = data_disk_count
        self.storage_config['data_disk_size'] =data_disk_size

    def set_cloud_config_from_arguments(self, arguments):
        self.set_cloud_config(username=arguments.username,
                              password=arguments.password,
                              rest_api_endpoint=arguments.rest_api_endpoint,
                              use_account_compute_creds=arguments.use_account_compute_creds,
                              storage_rest_api_endpoint=arguments.storage_rest_api_endpoint,
                              storage_username=arguments.storage_username,
                              storage_password=arguments.storage_password,
                              acl=arguments.acl,
                              ip_network=arguments.ip_network,
                              data_disk_count=arguments.count,
                              data_disk_size=arguments.size)

    def create_parser(self, argparser):

        # compute settings parser
        compute_config = argparser.add_argument_group("compute config settings")
        compute_creds = compute_config.add_mutually_exclusive_group()
        compute_creds.add_argument("--enable-account-compute-creds",
                                   dest="use_account_compute_creds",
                                   action="store_true",
                                   default=None,
                                   help="to use account compute credentials")
        compute_creds.add_argument("--disable-account-compute-creds",
                                   dest="use_account_compute_creds",
                                   action="store_false",
                                   default=None,
                                   help="to disable account compute credentials")
        compute_config.add_argument("--username",
                                    dest="username",
                                    default=None,
                                    help="username for opc cloud account")
        compute_config.add_argument("--password",
                                    dest="password",
                                    default=None,
                                    help="password for opc cloud account")
        compute_config.add_argument("--rest-api-endpoint",
                                    dest="rest_api_endpoint",
                                    default=None,
                                    help="Rest API endpoint for oracle opc account")

        # network settings parser
        network_config_group = argparser.add_argument_group("network config settings")
        network_config_group.add_argument("--acl",
                                          dest="acl",
                                          help="acl for opc", )
        network_config_group.add_argument("--ip-network",
                                          dest="ip_network",
                                          help="subnet name for opc")
        
        # storage config settings parser
        storage_config = argparser.add_argument_group("storage config settings")
        storage_config.add_argument("--storage-rest-api-endpoint",
                                    dest="storage_rest_api_endpoint",
                                    default=None,
                                    help="REST API endpoint for storage cloud")
        storage_config.add_argument("--storage-username",
                                    dest="storage_username",
                                    default=None,
                                    help="username for opc cloud account")
        storage_config.add_argument("--storage-password",
                                    dest="storage_password",
                                    default=None,
                                    help="password for opc cloud account")



"""
The Pipelines module contains the base definition for
a generic Pipelines commands.
"""
from qds_sdk.actions import *
import json
from qds_sdk.qubole import Qubole
from qds_sdk.resource import Resource
from argparse import ArgumentParser

log = logging.getLogger("qds_quest")

# Pattern matcher for s3 path
_URI_RE = re.compile(r's3://([^/]+)/?(.*)')


class PipelinesCmdLine:
    """qds_sdk.PipelinesCmdLine is the interface used by qds.py."""

    @staticmethod
    def parsers():
        argparser = ArgumentParser(prog="qds.py pipelines",
                                   description="Pipelines client for Qubole Data Service.")
        subparsers = argparser.add_subparsers()

        # Create
        create = subparsers.add_parser("create", help="Create a new pipeline")
        create.add_argument("--create-type", dest="create_type", required=True,
                            help="create_type=1 for assisted, "
                                 "create_type=2 for jar, create_type=3 for code")
        create.add_argument("--pipeline-name", dest="name", required=True,
                            help="Name of pipeline")
        create.add_argument("--description", dest="description", default=None,
                            help="Pipeline description"),
        create.add_argument("--cluster-label", dest="cluster_label",
                            default="default", help="Cluster label")
        create.add_argument("-c", "--code", dest="code", help="query string")
        create.add_argument("-f", "--script-location", dest="script_location",
                            help="Path where code to run is stored. "
                                 "local file path")
        create.add_argument("-l", "--language", dest="language",
                            help="Language for bring your own code, "
                                 "valid values are python and scala")
        create.add_argument("--jar-path", dest="jar_path",
                            help="Location of Jar")
        create.add_argument("--user-arguments", dest="user_arguments",
                            help="Additional user arguments")
        create.add_argument("--main-class-name", dest="main_class_name",
                            help="class name of your jar file. "
                                 "Required for create_type=2(BYOJ)")
        create.add_argument("--command-line-options",
                            dest="command_line_options",
                            help="command line options on property page.")
        create.set_defaults(func=PipelinesCmdLine.create)

        # Update/Edit
        update_properties = subparsers.add_parser("update-property",
                                                  help="Update properties of "
                                                       "a existing pipeline")
        update_properties.add_argument("--pipeline-id",
                                       dest="pipeline_id",
                                       required=True,
                                       help='Id of pipeline which need to be updated')
        update_properties.add_argument("--cluster-label", dest="cluster_label",
                                       help="Update cluster label.")
        update_properties.add_argument("--command-line-options", dest="command_line_options",
                                       help="command line options on property page.")
        update_properties.add_argument("--can-retry", dest="can_retry",
                                       help="can retry true or false")
        update_properties.set_defaults(func=PipelinesCmdLine.update_properties)
        update_code = subparsers.add_parser("update-code",
                                            help="Update code of a existing pipeline")
        update_code.add_argument(
            "-c", "--code", dest="code", help="query string")
        update_code.add_argument("-f", "--script-location", dest="script_location",
                                 help="Path where code to run is stored. local file path")
        update_code.set_defaults(func=PipelinesCmdLine.update_code)
        update_code.add_argument(
            "--jar-path",
            dest="jar_path",
            help="Location of Jar")
        update_code.add_argument("--user-arguments", dest="user_arguments",
                                 help="Additional user arguments")
        update_code.add_argument("--main-class-name", dest="main_class_name",
                                 help="class name of your jar file. "
                                      "Required for create_type=2(BYOJ)")
        update_code.add_argument("--language", dest="language",
                                 help="language of code scala or python")
        update_code.add_argument("--pipeline-id", dest="pipeline_id", required=True,
                                 help='Id of pipeline which need to be updated')

        # Pipeline Util (Utility for start, pause, clone, edit, delete,
        # archive)
        delete = subparsers.add_parser("delete", help="Delete Pipeline")
        delete.add_argument("--pipeline-id", dest="pipeline_id", required=True,
                            help='Id of pipeline which need to be started')
        delete.set_defaults(func=PipelinesCmdLine.delete)
        status = subparsers.add_parser("status", help="Status of Pipeline")
        status.add_argument("--pipeline-id", dest="pipeline_id", required=True,
                            help='Id of pipeline which need to be started')
        status.set_defaults(func=PipelinesCmdLine.status)
        start = subparsers.add_parser("start", help="Start Pipeline")
        start.add_argument("--pipeline-id", dest="pipeline_id", required=True,
                           help='Id of pipeline which need to be started')
        start.set_defaults(func=PipelinesCmdLine.start)
        pause = subparsers.add_parser("pause", help="pause Pipeline")
        pause.add_argument("--pipeline-id", dest="pipeline_id", required=True,
                           help='Id of pipeline which need to be started')
        pause.set_defaults(func=PipelinesCmdLine.pause)
        clone = subparsers.add_parser("clone", help="clone Pipeline")
        clone.add_argument("--pipeline-id", dest="pipeline_id", required=True,
                           help='Id of pipeline which need to be started')
        clone.set_defaults(func=PipelinesCmdLine.clone)
        archive = subparsers.add_parser("archive", help="archive Pipeline")
        archive.add_argument("--pipeline-id", dest="pipeline_id", required=True,
                             help='Id of pipeline which need to be started')
        archive.set_defaults(func=PipelinesCmdLine.archive)
        health = subparsers.add_parser("health", help="health of Pipeline")
        health.add_argument("--pipeline-id", dest="pipeline_id", required=True,
                            help='Id of pipeline which need to be started')
        health.set_defaults(func=PipelinesCmdLine.health)
        # list
        index = subparsers.add_parser("list", help="list of Pipeline.")
        index.add_argument("--pipeline-status", dest="status", required=True,
                           help='Id of pipeline which need to be started. '
                                'Valid values = [active, archive, all, draft] ')
        index.set_defaults(func=PipelinesCmdLine.index)
        return argparser

    @staticmethod
    def run(args):
        """
        Commandline method to run pipeline.
        :param args:
        :return:
        """
        parser = PipelinesCmdLine.parsers()
        parsed = parser.parse_args(args)
        return parsed.func(parsed)

    @staticmethod
    def delete(args):
        """
        Commandline method to delete pipeline.
        :param args:
        :return:
        """
        response = Pipelines.delete(args.pipeline_id)
        return json.dumps(
            response, default=lambda o: o.attributes, sort_keys=True, indent=4)

    @staticmethod
    def pause(args):
        """
        Commandline method to pause pipeline.
        :param args:
        :return:
        """
        response = Pipelines.pause(args.pipeline_id)
        return json.dumps(
            response, default=lambda o: o.attributes, sort_keys=True, indent=4)

    @staticmethod
    def archive(args):
        """
        commandline method to archive active pipeline.
        :param args:
        :return:
        """
        response = Pipelines.archive(args.pipeline_id)
        return json.dumps(
            response, default=lambda o: o.attributes, sort_keys=True, indent=4)

    @staticmethod
    def clone(args):
        """
        Commandline method to clone pipeline
        :param args:
        :return:
        """
        response = Pipelines.clone(args.pipeline_id)
        return json.dumps(response, default=lambda o: o.attributes, sort_keys=True, indent=4)

    @staticmethod
    def status(args):
        """
        CommandLine method to get pipeline status
        :param args:
        :return:
        """
        response = Pipelines.get_status(args.pipeline_id)
        return json.dumps(
            response, default=lambda o: o.attributes, sort_keys=True, indent=4)

    @staticmethod
    def health(args):
        """
        Commandline method to get health of pipeline.
        :param args:
        :return:
        """
        response = Pipelines.get_health(args.pipeline_id)
        return json.dumps(
            response, default=lambda o: o.attributes, sort_keys=True, indent=4)

    @staticmethod
    def start(args):
        """
        Commandline method to start pipeline.
        :param args:
        :return:
        """
        response = Pipelines.start(args.pipeline_id)
        return json.dumps(response, sort_keys=True, indent=4)

    @staticmethod
    def index(args):
        """
        Commandline method to list pipeline.
        :param args:
        :return:
        """
        pipelinelist = Pipelines.list(args.status)
        return json.dumps(
            pipelinelist, default=lambda o: o.attributes, sort_keys=True, indent=4)

    @staticmethod
    def create(args):
        """
        Commandline method to create pipeline.
        :param args:
        :return:
        """
        pipeline = None
        if int(args.create_type) == 2:
            pipeline = PipelinesJar.create_pipeline(pipeline_name=args.name,
                                                jar_path=args.jar_path,
                                                main_class_name=args.main_class_name,
                                                cluster_label=args.cluster_label,
                                                user_arguments=args.user_arguments,
                                                command_line_options=args.command_line_options)
        elif int(args.create_type) == 3:
            if args.code:
                pipeline = PipelinesCode.create_pipeline(pipeline_name=args.name,
                                                     cluster_label=args.cluster_label,
                                                     code=args.code,
                                                     file_path=args.script_location,
                                                     language=args.language,
                                                     user_arguments=args.user_arguments,
                                                     command_line_options=args.command_line_options)
            elif args.script_location:
                pipeline = PipelinesCode.create_pipeline(pipeline_name=args.name,
                                                     cluster_label=args.cluster_label,
                                                     code=args.code,
                                                     file_path=args.script_location,
                                                     language=args.language,
                                                     user_arguments=args.user_arguments,
                                                     command_line_options=args.command_line_options)

        return json.dumps(pipeline)

    @staticmethod
    def update_properties(args):
        """
        Commandline method to update pipeline properties.
        :param args:
        :return:
        """
        params = args.__dict__
        log.debug(params)
        Pipelines.add_property(pipeline_id=args.pipeline_id,
                           cluster_label=args.cluster_label,
                           can_retry=args.can_retry,
                           command_line_options=args.command_line_options)

    @staticmethod
    def update_code(args):
        """
        Commandline method to update code/Jar_Path
        :param args:
        :return:
        """
        if args.jar_path or args.main_class_name:
            response = PipelinesJar.save_code(pipeline_id=args.pipeline_id,
                                          code=args.code,
                                          file_path=args.script_location,
                                          language=args.language,
                                          jar_path=args.jar_path,
                                          user_arguments=args.user_arguments,
                                          main_class_name=args.main_class_name)
        elif args.code or args.script_location:
            response = PipelinesCode.save_code(pipeline_id=args.pipeline_id,
                                           code=args.code,
                                           file_path=args.script_location,
                                           language=args.language,
                                           jar_path=args.jar_path,
                                           user_arguments=args.user_arguments,
                                           main_class_name=args.main_class_name)
        return json.dumps(response, sort_keys=True, indent=4)


class Pipelines(Resource):
    """qds_sdk.Pipelines is the base Qubole Pipelines class."""

    """ all commands use the /pipelines endpoint"""

    rest_entity_path = "pipelines"
    pipeline_id = None
    pipeline_name = None
    pipeline_code = None
    jar_path = None

    @staticmethod
    def get_pipline_id(response):
        return str(response.get('data').get('id'))

    @staticmethod
    def list(status=None):
        """
        Method to list pipeline on the basis of status.
        :param status: Valid values - all, draft, archive, active.
        :return: List of pipeline in json format.
        """
        if status is None or status.lower() == 'all':
            params = {"filter": "draft,archive,active"}
        else:
            params = {"filter": status.lower()}
        conn = Qubole.agent()
        url_path = Pipelines.rest_entity_path
        pipeline_list = conn.get(url_path, params)
        return pipeline_list

    @classmethod
    def create(cls, pipeline_name, create_type, **kwargs):
        """
        Create a pipeline object by issuing a POST
        request to the /pipeline?mode=wizard endpoint
        Note - this creates pipeline in draft mode

        Args:
            pipeline_name: Name to be given.
            create_type: 1->Assisted, 2->Code, 3->Jar
            **kwargs: keyword arguments specific to create type

        Returns:
            response
        """
        conn = Qubole.agent()
        data = {"data": {
            "attributes":
                {"name": pipeline_name, "status": "DRAFT",
                 "create_type": create_type},
            "type": "pipelines"}
        }
        url = Pipelines.rest_entity_path + "?mode=wizard"
        response = conn.post(url, data)
        cls.pipeline_id = Pipelines.get_pipline_id(response)
        cls.pipeline_name = pipeline_name

    @staticmethod
    def start(pipeline_id):
        """
        Method to start Pipeline
        :param pipeline_id: id of pipeline to be deleted
        :return: response
        """
        conn = Qubole.agent()
        url = Pipelines.rest_entity_path + "/" + pipeline_id + "/start"
        response = conn.put(url)
        pipeline_status = Pipelines.get_status(pipeline_id)
        while pipeline_status == 'waiting':
            log.info("Pipeline is in waiting state....")
            time.sleep(10)
            pipeline_status = response.get(
                'data').get('pipeline_instance_status')
        log.debug("State of pipeline is %s", pipeline_status)
        return response

    @staticmethod
    def add_property(pipeline_id,
                     cluster_label,
                     checkpoint_location=None,
                     output_mode=None,
                     trigger_interval=None,
                     can_retry=True,
                     command_line_options=None):
        """
        Method to add properties in pipeline
        :param can_retry:
        :param pipeline_id:
        :param cluster_label:
        :param checkpoint_location:
        :param trigger_interval:
        :param output_mode:
        :param command_line_options:
        :return:
        """
        conn = Qubole.agent()
        if command_line_options is None:
            command_line_options = """--conf spark.driver.extraLibraryPath=/usr/lib/hadoop2/lib/native\n--conf spark.eventLog.compress=true\n--conf spark.eventLog.enabled=true\n--conf spark.sql.streaming.qubole.enableStreamingEvents=true\n--conf spark.qubole.event.enabled=true"""
        data = {"data": {"attributes": {
            "cluster_label": cluster_label,
            "can_retry": can_retry,
            "checkpoint_location": checkpoint_location,
            "trigger_interval": trigger_interval,
            "output_mode": output_mode,
            "command_line_options": command_line_options
        },
            "type": "pipeline/properties"
        }
        }
        log.info("Data {}".format(data))
        url = Pipelines.rest_entity_path + "/" + pipeline_id + "/properties"
        response = conn.put(url, data)
        log.debug(response)
        return response

    @classmethod
    def save_code(cls, pipeline_id,
                  code=None,
                  file_path=None,
                  language=None,
                  jar_path=None,
                  main_class_name=None,
                  user_arguments=None):
        """
        :param file_path:
        :param code:
        :param language:
        :param user_arguments:
        :param pipeline_id:
        :param jar_path:
        :param main_class_name:
        :return:
        """
        data = None
        if cls.create_type == 2:
            if jar_path is None or main_class_name is None:
                raise ParseError(
                    "Provide Jar path for BYOJ mode.")
            else:
                cls.jar_path = jar_path
                data = {"data": {
                    "attributes": {"create_type": cls.create_type,
                                   "user_arguments": str(user_arguments),
                                   "jar_path": str(jar_path),
                                   "main_class_name": str(main_class_name)}}}

        elif cls.create_type == 3:
            if code or file_path:
                try:
                    if file_path:
                        with open(file_path, 'r') as f:
                            code = f.read()
                    else:
                        code = code
                except IOError as e:
                    raise ParseError("Unable to open script location or script "
                                     "location and code both are empty. ",
                                     e.message)
                cls.pipeline_code = code
                data = {"data": {
                    "attributes": {"create_type": cls.create_type, "user_arguments": str(user_arguments),
                                   "code": str(code), "language": str(language)}}}

            else:
                raise ParseError(
                    "Provide code or file location for BYOC mode.")

        conn = Qubole.agent()
        url = cls.rest_entity_path + "/" + str(pipeline_id) + "/save_code"
        response = conn.put(url, data)
        log.debug(response)
        return response

    @staticmethod
    def get_health(pipeline_id):
        """
        Get Pipeline Health
        :param pipeline_id:
        :return:
        """
        conn = Qubole.agent()
        url = Pipelines.rest_entity_path + "/" + pipeline_id
        response = conn.get(url)
        log.info(response)
        return response.get("data").get("attributes").get("health")

    @staticmethod
    def clone(pipeline_id):
        """
        Method to clone pipeline
        :param pipeline_id:
        :return:
        """
        url = Pipelines.rest_entity_path + "/" + pipeline_id + "/duplicate"
        log.info("Cloning pipeline with id {}".format(pipeline_id))
        conn = Qubole.agent()
        return conn.post(url)

    @staticmethod
    def pause(pipeline_id):
        """
        Method to pause pipeline
        :param pipeline_id:
        :return:
        """
        url = Pipelines.rest_entity_path + "/" + pipeline_id + "/pause"
        log.info("Pausing pipeline with id {}".format(pipeline_id))
        conn = Qubole.agent()
        return conn.put(url)

    @staticmethod
    def archive(pipeline_id):
        """
        Method to Archive pipeline
        :param pipeline_id:
        :return:
        """
        url = Pipelines.rest_entity_path + "/" + pipeline_id + "/archive"
        log.info("Archiving pipeline with id {}".format(pipeline_id))
        conn = Qubole.agent()
        return conn.put(url)

    @staticmethod
    def get_status(pipeline_id):
        """
        Get pipeline status
        :param pipeline_id:
        :return:
        """
        conn = Qubole.agent()
        url = Pipelines.rest_entity_path + "/" + pipeline_id
        response = conn.get(url)
        log.debug(response)
        return response.get("data").get(
            "attributes").get("pipeline_instance_status")

    @staticmethod
    def delete(pipeline_id):
        """
        Method to delete pipeline
        :param pipeline_id:
        :return:
        """
        conn = Qubole.agent()
        url = Pipelines.rest_entity_path + "/" + pipeline_id + "/delete"
        log.info("Deleting Pipeline with id: {}".format(pipeline_id))
        response = conn.put(url)
        log.info(response)
        return response

    @staticmethod
    def edit_pipeline_name(pipeline_id, pipeline_name):
        """
        Method to edit pipeline name (Required in case of cloning)
        :param pipeline_id:
        :param pipeline_name:
        :return:
        """
        conn = Qubole.agent()
        url = Pipelines.rest_entity_path + "/" + pipeline_id
        data = {
            "data": {
                "attributes": {
                    "name": pipeline_name},
                "type": "pipelines"}}
        return conn.put(url, data)

    @staticmethod
    def set_alert(pipeline_id, channel_id):
        """

        :param pipeline_id:
        :param channel_id: List of channel's id
        :return:
        """
        data = {
            "data": {"attributes": {
                "event_type": "error",
                "notification_channels": [channel_id],
                "can_notify": True},
                "type": "pipeline/alerts"
            }
        }
        conn = Qubole.agent()
        url = Pipelines.rest_entity_path + "/" + pipeline_id + "/alerts"
        return conn.put(url, data)

    @staticmethod
    def get_code(pipeline_id):
        """
        Get pipeline code
        :param pipeline_id:
        :return:
        """
        url = Pipelines.rest_entity_path + "/" + pipeline_id
        conn = Qubole.agent()
        reponse = conn.get(url)
        code = reponse.get("meta")["command_details"]["code"]
        return code


class PipelinesCode(Pipelines):
    create_type = 3

    @staticmethod
    def create_pipeline(pipeline_name,
                        cluster_label,
                        code=None,
                        file_path=None,
                        language=None,
                        can_retry=True,
                        channel_id=None,
                        command_line_options=None,
                        user_arguments=None):
        """
        Method to create pipeline in BYOC mode in one go.
        :param file_path:
        :param code:
        :param command_line_options:
        :param user_arguments:
        :param pipeline_name:
        :param cluster_label:
        :param language:
        :param can_retry:
        :param channel_id:
        :return:
        """
        PipelinesCode.create(pipeline_name, PipelinesCode.create_type)
        pipeline_id = PipelinesCode.pipeline_id
        response = PipelinesCode.add_property(pipeline_id, cluster_label,
                                          can_retry=can_retry,
                                          command_line_options=command_line_options)
        log.debug(response)
        response = PipelinesCode.save_code(pipeline_id,
                                       code=code,
                                       file_path=file_path,
                                       language=language,
                                       user_arguments=user_arguments)
        if channel_id:
            response = Pipelines.set_alert(pipeline_id, channel_id)
            log.info(response)
        return response


class PipelinesJar(Pipelines):
    create_type = 2

    @staticmethod
    def create_pipeline(pipeline_name,
                        jar_path,
                        cluster_label,
                        main_class_name,
                        channel_id=None,
                        can_retry=True,
                        command_line_options=None,
                        user_arguments=None):
        """
        Method to create pipeline in BYOJ mode
        :param pipeline_name:
        :param jar_path:
        :param cluster_label:
        :param main_class_name:
        :param channel_id:
        :param can_retry:
        :param command_line_options:
        :param user_arguments:
        :return:
        """
        PipelinesJar.create(pipeline_name, PipelinesJar.create_type)
        pipeline_id = PipelinesJar.pipeline_id
        PipelinesJar.add_property(pipeline_id,
                              cluster_label,
                              can_retry=can_retry,
                              command_line_options=command_line_options)
        PipelinesJar.save_code(pipeline_id,
                           jar_path=jar_path,
                           main_class_name=main_class_name,
                           user_arguments=user_arguments)
        PipelinesJar.jar_path = jar_path
        if channel_id:
            response = Pipelines.set_alert(pipeline_id, channel_id)
            log.info(response)
        return PipelinesJar


class PipelinesAssisted(Pipelines):
    create_type = 1

    @staticmethod
    def add_source():
        """Method to add source."""
        pass

    @staticmethod
    def add_sink():
        """Method to add sink."""
        pass

    @staticmethod
    def create_pipeline():
        """Parent Method to create end to end pipeline."""
        pass

    @staticmethod
    def add_operator():
        """Parent method to add operator"""
        pass

    @staticmethod
    def _select_operator():
        """Method to add select operator."""
        pass

    @staticmethod
    def _filter_operator():
        """Method to add filter operator."""
        pass

    @staticmethod
    def _watermark_operator():
        """Method to add watermark operator"""
        pass

    @staticmethod
    def _window_group_operator():
        """Method to add window group operator"""
        pass

    @staticmethod
    def _source_kafka():
        """Method to as kafka as source."""
        pass

    @staticmethod
    def _source_kinesis():
        """Method to add kinesis as source."""
        pass

    @staticmethod
    def _source_s3():
        """Method to add s3 as source."""
        pass

    @staticmethod
    def _source_google_storage():
        """Method to add google storage as source."""
        pass

    @staticmethod
    def _sink_kafka():
        """Method to add kafka as sink."""
        pass

    @staticmethod
    def _sink_s3():
        """Method to add s3 as sink."""
        pass

    @staticmethod
    def _sink_hive():
        """method to add hive as sink."""
        pass

    @staticmethod
    def _sink_snowflake():
        """Method to add Snowflake as sink"""
        pass

    @staticmethod
    def _sink_google_storage():
        """Method to add google storage as sink"""
        pass

    @staticmethod
    def _sink_BigQuery():
        """Method to add BigQuery as sink."""
        pass

    @staticmethod
    def add_registry():
        """Method to add registry."""
        pass

    @staticmethod
    def switch_from_assisted():
        """Method to switch to Assisted from BYOC or BYOJ mode."""
        pass



from qds_sdk.qubole import Qubole
from qds_sdk.resource import Resource
import argparse
import json


class UserCmdLine:
    @staticmethod
    def parsers():
        argparser = argparse.ArgumentParser(
            prog="qds.py user",
            description="User level operations for Qubole Data Service.")
        subparsers = argparser.add_subparsers(title="user operations")

        # To invite a new user
        invite = subparsers.add_parser(
            "invite", help="Invite a new user")
        invite.add_argument(
            "--email", dest="invitee_email", required=True,
            help="Email address of the new user to be added to the Qubole account")
        invite.add_argument(
            "--account-id", dest="account", required=True,
            help="Account ID of the current user")
        invite.add_argument(
            "--groups", dest="groups", required=False,
            help="Add groups to the new user, defaults to system-user")
        invite.set_defaults(func=UserCmdLine.invite)

        # enable qbol user
        enable = subparsers.add_parser(
            "enable", help="Enable a qbol user")
        enable.add_argument(
            "--qbol-user-id", dest="qbol_user_id", required=True,
            help="ID of the qbol_user, who should be enabled")
        enable.set_defaults(func=UserCmdLine.enable)

        # disable qbol user
        disable = subparsers.add_parser(
            "disable", help="Disable a qbol user")
        disable.add_argument(
            "--qbol-user-id", dest="qbol_user_id", required=True,
            help="ID of the qbol_user, who should be disabled")
        disable.set_defaults(func=UserCmdLine.disable)

        return argparser

    @staticmethod
    def run(args):
        parser = UserCmdLine.parsers()
        parsed = parser.parse_args(args)
        return parsed.func(parsed)

    @staticmethod
    def invite(args):
        data = vars(args)
        data.pop("func")
        result = User.invite("invite_new", data)
        return json.dumps(result, indent=4)

    @staticmethod
    def enable(args):
        data = vars(args)
        data.pop("func")
        result = Accounts.enable_disable("enable_qbol_user", data)
        return json.dumps(result, indent=4)

    @staticmethod
    def disable(args):
        data = vars(args)
        data.pop("func")
        result = Accounts.enable_disable("disable_qbol_user", data)
        return json.dumps(result, indent=4)


class User(Resource):

    rest_entity_path = "users"

    @classmethod
    def invite(cls, path, data):
        conn = Qubole.agent()
        return conn.post(cls.element_path(path), data)


class Accounts(Resource):
    rest_entity_path = "accounts"

    @classmethod
    def enable_disable(cls, path, data):
        conn = Qubole.agent()
        return conn.post(cls.element_path(path), data)



from qds_sdk.cloud.cloud import Cloud
import json
import ast

class OracleBmcCloud(Cloud):
    '''
    qds_sdk.cloud.OracleBmcCloud is the class which stores information about oracle bmc cloud config settings.
    The objects of this class can be used to set oracle_bmc cloud_config settings while create/update/clone a cluster.
    '''

    def __init__(self):
        self.compute_config = {}
        self.location = {}
        self.network_config = {}
        self.storage_config = {}

    def set_cloud_config(self,
                         compute_tenant_id=None,
                         compute_user_id=None,
                         compute_key_finger_print=None,
                         compute_api_private_rsa_key=None,
                         use_account_compute_creds=None,
                         subnet_id=None,
                         oracle_region=None,
                         oracle_availability_domain=None,
                         compartment_id=None,
                         image_id=None,
                         vcn_id=None,
                         availability_domain_info_map=None,
                         storage_tenant_id=None,
                         storage_user_id=None,
                         storage_key_finger_print=None,
                         storage_api_private_rsa_key=None,
                         block_volume_count=None,
                         block_volume_size=None):
        '''

        Args:
            compute_tenant_id: compute tenant id for oracle cluster

            compute_user_id: compute user id for oracle cluster

            compute_key_finger_print: compute key fingerprint for oracle cluster

            compute_api_private_rsa_key: compute api private rsa key for oracle cluster

            use_account_compute_creds: Set it to true to use the account's compute
                credentials for all clusters of the account.The default value is false

            subnet_id: subnet id for oracle

            oracle_region: region to create the cluster in

            oracle_availability_domain: availability zone to create the cluster in

            compartment_id: compartment id for oracle cluster

            image_id: image id for oracle cloud

            vcn_id: vcn to create the cluster in

            availability_domain_info_map: availability domain and subnet mapping for the cluster

            storage_tenant_id: tenant id for oracle cluster

            storage_user_id: storage user id for oracle cluster

            storage_key_finger_print: storage key fingerprint for oracle cluster

            storage_api_private_rsa_key: storage api private rsa key for oracle cluster

            block_volume_count: count of block volumes to be mounted to an instance as reserved disks

            block_volume_size: It is the size (in GB) of each block volume to be mounted to an instance as reserved disk

        '''

        self.set_compute_config(use_account_compute_creds, compute_tenant_id,
                                compute_user_id, compute_key_finger_print,
                                compute_api_private_rsa_key)
        self.set_location(oracle_region, oracle_availability_domain)
        self.set_network_config(vcn_id, subnet_id,
                                compartment_id, image_id, availability_domain_info_map)
        self.set_storage_config(storage_tenant_id, storage_user_id,
                                storage_key_finger_print, storage_api_private_rsa_key, block_volume_count,
                                block_volume_size)

    def  set_compute_config(self,
                            use_account_compute_creds=None,
                            compute_tenant_id=None,
                            compute_user_id=None,
                            compute_key_finger_print=None,
                            compute_api_private_rsa_key=None):
        self.compute_config['use_account_compute_creds'] = use_account_compute_creds
        self.compute_config['compute_tenant_id'] = compute_tenant_id
        self.compute_config['compute_user_id'] = compute_user_id
        self.compute_config['compute_key_finger_print'] = compute_key_finger_print
        self.compute_config['compute_api_private_rsa_key'] = compute_api_private_rsa_key

    def set_location(self,
                     oracle_region=None,
                     oracle_availability_domain=None):
        self.location['region'] = oracle_region
        self.location['availability_domain'] = oracle_availability_domain

    def set_network_config(self,
                           vcn_id=None,
                           subnet_id=None,
                           compartment_id=None,
                           image_id=None,
                           availability_domain_info_map=None):
        self.network_config['vcn_id'] = vcn_id
        self.network_config['subnet_id'] = subnet_id
        self.network_config['compartment_id'] = compartment_id
        self.network_config['image_id'] = image_id
        if availability_domain_info_map:
            self.network_config['availability_domain_info_map'] = availability_domain_info_map

    def set_storage_config(self,
                           storage_tenant_id=None,
                           storage_user_id=None,
                           storage_key_finger_print=None,
                           storage_api_private_rsa_key=None,
                           block_volume_count=None,
                           block_volume_size=None):
        self.storage_config['storage_tenant_id'] = storage_tenant_id
        self.storage_config['storage_user_id'] = storage_user_id
        self.storage_config['storage_key_finger_print'] = storage_key_finger_print
        self.storage_config['storage_api_private_rsa_key'] = storage_api_private_rsa_key
        self.storage_config['block_volume_count'] = block_volume_count
        self.storage_config['block_volume_size'] = block_volume_size

    def set_cloud_config_from_arguments(self, arguments):
        if arguments.availability_domain_info_map:
            try:
                arguments.availability_domain_info_map = ast.literal_eval(arguments.availability_domain_info_map)
                assert isinstance(arguments.availability_domain_info_map, list)
            except Exception as e:
                raise Exception("Invalid List format for availability_domain_info_map: %s" % e.message)
        self.set_cloud_config(compute_tenant_id=arguments.compute_tenant_id,
                              compute_user_id=arguments.compute_user_id,
                              compute_key_finger_print=arguments.compute_key_finger_print,
                              compute_api_private_rsa_key=arguments.compute_api_private_rsa_key,
                              use_account_compute_creds=arguments.use_account_compute_creds,
                              subnet_id=arguments.subnet_id,
                              oracle_region=arguments.region,
                              oracle_availability_domain=arguments.availability_domain,
                              compartment_id=arguments.compartment_id,
                              image_id=arguments.image_id,
                              vcn_id=arguments.vcn_id,
                              availability_domain_info_map=arguments.availability_domain_info_map,
                              storage_tenant_id=arguments.storage_tenant_id,
                              storage_user_id=arguments.storage_user_id,
                              storage_key_finger_print=arguments.storage_key_finger_print,
                              storage_api_private_rsa_key=arguments.storage_api_private_rsa_key,
                              block_volume_count=arguments.block_volume_count,
                              block_volume_size=arguments.block_volume_size
                              )

    def create_parser(self, argparser):
        # compute settings parser
        compute_config = argparser.add_argument_group("compute config settings")
        compute_creds = compute_config.add_mutually_exclusive_group()
        compute_creds.add_argument("--enable-account-compute-creds",
                                   dest="use_account_compute_creds",
                                   action="store_true",
                                   default=None,
                                   help="to use account compute credentials")
        compute_creds.add_argument("--disable-account-compute-creds",
                                   dest="use_account_compute_creds",
                                   action="store_false",
                                   default=None,
                                   help="to disable account compute credentials")
        compute_config.add_argument("--compute-tenant-id",
                                    dest="compute_tenant_id",
                                    default=None,
                                    help="tenant id for oracle cluster")
        compute_config.add_argument("--compute-user-id",
                                    dest="compute_user_id",
                                    default=None,
                                    help="compute user id for oracle cluster")
        compute_config.add_argument("--compute-key-finger-print",
                                    dest="compute_key_finger_print",
                                    default=None,
                                    help="compute key fingerprint for oracle cluster")
        compute_config.add_argument("--compute-api-private-rsa-key",
                                    dest="compute_api_private_rsa_key",
                                    default=None,
                                    help="compute api private rsa key for oracle cluster")

        # location settings parser
        location_group = argparser.add_argument_group("location config settings")
        location_group.add_argument("--oracle-region",
                                    dest="region",
                                    help="region to create the cluster in", )
        location_group.add_argument("--oracle-availability-zone",
                                    dest="availability_domain",
                                    help="availability zone to" +
                                         " create the cluster in", )

        # network settings parser
        network_config_group = argparser.add_argument_group("network config settings")
        network_config_group.add_argument("--compartment-id",
                                          dest="compartment_id",
                                          help="compartment id for oracle cluster")
        network_config_group.add_argument("--image-id",
                                          dest="image_id",
                                          help="image id for oracle cloud")
        network_config_group.add_argument("--vcn-id",
                                          dest="vcn_id",
                                          help="vcn to create the cluster in", )
        network_config_group.add_argument("--subnet-id",
                                          dest="subnet_id",
                                          help="subnet id for oracle")
        network_config_group.add_argument("--availability-domain-info-map",
                                          dest="availability_domain_info_map",
                                          help="availability domain and subnet mapping for the cluster")

        # storage config settings parser
        storage_config = argparser.add_argument_group("storage config settings")
        storage_config.add_argument("--storage-tenant-id",
                                    dest="storage_tenant_id",
                                    default=None,
                                    help="tenant id for oracle cluster")
        storage_config.add_argument("--storage-user-id",
                                    dest="storage_user_id",
                                    default=None,
                                    help="storage user id for oracle cluster")
        storage_config.add_argument("--storage-key-finger-print",
                                    dest="storage_key_finger_print",
                                    default=None,
                                    help="storage key fingerprint for oracle cluster")
        storage_config.add_argument("--storage-api-private-rsa-key",
                                    dest="storage_api_private_rsa_key",
                                    default=None,
                                    help="storage api private rsa key for oracle cluster")
        storage_config.add_argument("--block-volume-count",
                                    dest="block_volume_count",
                                    default=None,
                                    help="count of block volumes to be mounted to an instance as reserved disks",
                                    type=int)
        storage_config.add_argument("--block-volume-size",
                                    dest="block_volume_size",
                                    default=None,
                                    help="size (in GB) of each block volume to be mounted to an instance",
                                    type=int)



from qds_sdk.cloud.cloud import Cloud
class GcpCloud(Cloud):
    '''
    qds_sdk.cloud.GcpCloud is the class which stores information about gcp cloud config settings.
    The objects of this class can be use to set gcp cloud_config settings while create/update/clone a cluster.
    '''

    def __init__(self):
        self.compute_config = {}
        self.location = {}
        self.network_config = {}
        self.storage_config = {}
        self.cluster_composition = {}

    def set_cloud_config(self,
                         qsa_client_id=None,
                         customer_project_id=None,
                         qsa_client_email=None,
                         qsa_private_key_id=None,
                         qsa_private_key=None,
                         comp_client_email=None,
                         inst_client_email=None,
                         use_account_compute_creds=None,
                         gcp_region=None,
                         gcp_zone=None,
                         storage_disk_size_in_gb=None,
                         storage_disk_count=None,
                         storage_disk_type=None,
                         bastion_node_public_dns=None,
                         vpc_id=None,
                         subnet_id=None,
                         master_preemptible=None,
                         min_nodes_preemptible=None,
                         min_nodes_preemptible_percentage=None,
                         autoscaling_nodes_preemptible=None,
                         autoscaling_nodes_preemptible_percentage=None):
        '''

        Args:
            qsa_client_id: Compute client id for gcp cluster

            customer_project_id: Compute project id for gcp cluster

            qsa_client_email: Compute client email for gcp cluster

            qsa_private_key_id: Compute private key id for gcp cluster

            qsa_private_key: Compute private key for gcp cluster

            comp_client_email: Client compute service account email
            
            inst_client_email: Client storage/instance service account email

            use_account_compute_creds: Set it to true to use the account's compute
                credentials for all clusters of the account.The default value is false

            gcp_region: Region for gcp cluster

            bastion_node_public_dns: public dns name of the bastion node.
                Required only if cluster is in a private subnet.

            vpc_id: Vpc id for gcp cluster

            subnet_id: Subnet id for gcp cluster

            master_preemptible: if the master node is preemptible

            min_nodes_preemptible: if the min nodes are preemptible

            min_nodes_preemptible_percentage: percentage of min nodes that are preemptible

            autoscaling_nodes_preemptible: if the autoscaling nodes are preemptible

            autoscaling_nodes_preemptible_percentage: percentage of autoscaling nodes that are preemptible
        '''

        self.set_compute_config(use_account_compute_creds, qsa_client_id, customer_project_id, qsa_client_email,
                                qsa_private_key_id, qsa_private_key, comp_client_email)
        self.set_location(gcp_region, gcp_zone)
        self.set_network_config(bastion_node_public_dns, vpc_id, subnet_id)
        self.set_storage_config(inst_client_email, storage_disk_size_in_gb, storage_disk_count, storage_disk_type)
        self.set_cluster_composition(master_preemptible, min_nodes_preemptible, min_nodes_preemptible_percentage,
                                     autoscaling_nodes_preemptible, autoscaling_nodes_preemptible_percentage)

    def set_compute_config(self,
                           use_account_compute_creds=None,
                           qsa_client_id=None,
                           customer_project_id=None,
                           qsa_client_email=None,
                           qsa_private_key_id=None,
                           qsa_private_key=None,
                           comp_client_email=None):
        self.compute_config['use_account_compute_creds'] = use_account_compute_creds
        self.compute_config['qsa_client_id'] = qsa_client_id
        self.compute_config['customer_project_id'] = customer_project_id
        self.compute_config['qsa_client_email'] = qsa_client_email
        self.compute_config['qsa_private_key_id'] = qsa_private_key_id
        self.compute_config['qsa_private_key'] = qsa_private_key
        self.compute_config['comp_client_email'] = comp_client_email

    def set_location(self,
                     gcp_region=None,
                     gcp_zone=None,
                     ):
        self.location['region'] = gcp_region
        self.location['zone'] = gcp_zone

    def set_network_config(self,
                           bastion_node_public_dns=None,
                           vpc_id=None,
                           subnet_id=None):
        self.network_config['bastion_node_public_dns'] = bastion_node_public_dns
        self.network_config['network'] = vpc_id
        self.network_config['subnet'] = subnet_id

    def set_storage_config(self,
                           inst_client_email=None,
                           storage_disk_size_in_gb=None,
                           storage_disk_count=None,
                           storage_disk_type=None
                           ):
        self.storage_config['inst_client_email'] = inst_client_email
        self.storage_config['disk_size_in_gb'] = storage_disk_size_in_gb
        self.storage_config['disk_count'] = storage_disk_count
        self.storage_config['disk_type'] = storage_disk_type

    def set_cluster_composition(self,
                                master_preemptible=None,
                                min_nodes_preemptible=None,
                                min_nodes_preemptible_percentage=None,
                                autoscaling_nodes_preemptible=None,
                                autoscaling_nodes_preemptible_percentage=None):
        self.cluster_composition['master'] = {}
        self.cluster_composition['master']['preemptible'] = master_preemptible
        self.cluster_composition['min_nodes'] = {}
        self.cluster_composition['min_nodes']['preemptible'] = min_nodes_preemptible
        self.cluster_composition['min_nodes']['percentage'] = min_nodes_preemptible_percentage
        self.cluster_composition['autoscaling_nodes'] = {}
        self.cluster_composition['autoscaling_nodes']['preemptible'] = autoscaling_nodes_preemptible
        self.cluster_composition['autoscaling_nodes']['percentage'] = autoscaling_nodes_preemptible_percentage

    def set_cloud_config_from_arguments(self, arguments):
        self.set_cloud_config(qsa_client_id=arguments.qsa_client_id,
                              customer_project_id=arguments.customer_project_id,
                              qsa_client_email=arguments.qsa_client_email,
                              qsa_private_key_id=arguments.qsa_private_key_id,
                              qsa_private_key=arguments.qsa_private_key,
                              inst_client_email=arguments.inst_client_email,
                              comp_client_email=arguments.comp_client_email,
                              use_account_compute_creds=arguments.use_account_compute_creds,
                              gcp_region=arguments.gcp_region,
                              gcp_zone=arguments.gcp_zone,
                              storage_disk_size_in_gb=arguments.storage_disk_size_in_gb,
                              storage_disk_count=arguments.storage_disk_count,
                              storage_disk_type=arguments.storage_disk_type,
                              bastion_node_public_dns=arguments.bastion_node_public_dns,
                              vpc_id=arguments.vpc_id,
                              subnet_id=arguments.subnet_id,
                              master_preemptible=arguments.master_preemptible,
                              min_nodes_preemptible=arguments.min_nodes_preemptible,
                              min_nodes_preemptible_percentage=arguments.min_nodes_preemptible_percentage,
                              autoscaling_nodes_preemptible=arguments.autoscaling_nodes_preemptible,
                              autoscaling_nodes_preemptible_percentage=arguments.autoscaling_nodes_preemptible_percentage)

    def create_parser(self, argparser):
        # compute settings parser
        compute_config = argparser.add_argument_group("compute config settings")
        compute_creds = compute_config.add_mutually_exclusive_group()
        compute_creds.add_argument("--enable-account-compute-creds",
                                   dest="use_account_compute_creds",
                                   action="store_true",
                                   default=None,
                                   help="to use account compute credentials")
        compute_creds.add_argument("--disable-account-compute-creds",
                                   dest="use_account_compute_creds",
                                   action="store_false",
                                   default=None,
                                   help="to disable account compute credentials")
        compute_config.add_argument("--qsa-client-id",
                                    dest="qsa_client_id",
                                    default=None,
                                    help="qsa client id for gcp cluster")
        compute_config.add_argument("--customer-project-id",
                                    dest="customer_project_id",
                                    default=None,
                                    help="customer project id for gcp cluster")
        compute_config.add_argument("--qsa-client-email",
                                    dest="qsa_client_email",
                                    default=None,
                                    help="qsa client email for gcp cluster")
        compute_config.add_argument("--qsa-private-key-id",
                                    dest="qsa_private_key_id",
                                    default=None,
                                    help="qsa private key id for gcp cluster")
        compute_config.add_argument("--qsa-private-key",
                                    dest="qsa_private_key",
                                    default=None,
                                    help="qsa private key for gcp cluster")
        compute_config.add_argument("--compute-client-email",
                                    dest="comp_client_email",
                                    default=None,
                                    help="client compute service account email")

        # location settings parser
        location_group = argparser.add_argument_group("location config settings")
        location_group.add_argument("--gcp-region",
                                    dest="gcp_region",
                                    help="region to create the cluster in")
        location_group.add_argument("--gcp-zone",
                                    dest="gcp_zone",
                                    help="zone to create the cluster in")

        # network settings parser
        network_config_group = argparser.add_argument_group("network config settings")
        network_config_group.add_argument("--bastion-node-public-dns",
                                          dest="bastion_node_public_dns",
                                          help="public dns name of the bastion node. Required only if cluster is in private subnet")
        network_config_group.add_argument("--vpc-id",
                                          dest="vpc_id",
                                          help="vpc id to create the cluster in")
        network_config_group.add_argument("--subnet-id",
                                          dest="subnet_id",
                                          help="subnet id to create the cluster in")

        # storage config settings parser
        storage_config = argparser.add_argument_group("storage config settings")

        storage_config.add_argument("--storage-client-email",
                                    dest="inst_client_email",
                                    default=None,
                                    help="client storage service account email")
        storage_config.add_argument("--storage-disk-size-in-gb",
                                    dest="storage_disk_size_in_gb",
                                    default=None,
                                    help="disk size in gb for gcp cluster")
        storage_config.add_argument("--storage-disk-count",
                                    dest="storage_disk_count",
                                    default=None,
                                    help="disk count for gcp cluster")
        storage_config.add_argument("--storage-disk-type",
                                    dest="storage_disk_type",
                                    default=None,
                                    help="disk type for gcp cluster")
        # cluster composition settings parser
        cluster_composition = argparser.add_argument_group("cluster composition settings")
        cluster_composition.add_argument("--master-preemptible",
                                         dest="master_preemptible",
                                         action="store_true",
                                         default=None,
                                         help="if the master node is preemptible")
        cluster_composition.add_argument("--min-nodes-preemptible",
                                         dest="min_nodes_preemptible",
                                         action="store_true",
                                         default=None,
                                         help="if the min nodes are preemptible")
        cluster_composition.add_argument("--min-nodes-preemptible-percentage",
                                         dest="min_nodes_preemptible_percentage",
                                         type=int,
                                         default=None,
                                         help="percentage of min nodes that are preemptible")
        cluster_composition.add_argument("--autoscaling-nodes-preemptible",
                                         dest="autoscaling_nodes_preemptible",
                                         action="store_true",
                                         default=None,
                                         help="if the autoscaling nodes are preemptible")
        cluster_composition.add_argument("--autoscaling-nodes-preemptible-percentage",
                                         dest="autoscaling_nodes_preemptible_percentage",
                                         type=int,
                                         default=None,
                                         help="percentage of autoscaling nodes that are preemptible")
